# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from functionaltests.client import base
from functionaltests.common import cleanup
from functionaltests import utils
from testtools import testcase

from barbicanclient import exceptions


create_secret_defaults_data = {
    "name": "AES key",
    "expiration": "2030-02-28T19:14:44.180394",
    "algorithm": "aes",
    "bit_length": 256,
    "mode": "cbc",
    "payload": b"gF6+lLoF3ohA9aPRpt+6bQ==",
    "payload_content_type": "application/octet-stream",
    "payload_content_encoding": "base64",
}

create_container_defaults_data = {
    "name": "containername",
    "secrets": {}
}

create_container_rsa_data = {
    "name": "rsacontainer",
}

create_container_empty_data = {
    "name": None,
    "secrets": {}
}

accepted_str_values = {
    'alphanumeric': ['a2j3j6ll9'],
    'punctuation': ['~!@#$%^&*()_+`-={}[]|:;<>,.?'],
    'len_255': ['a' * 255],
    'uuid': ['54262d9d-4bc7-4821-8df0-dc2ca8e112bb'],
    'empty': ['']
}


@utils.parameterized_test_case
class BaseContainersTestCase(base.TestCase):

    def setUp(self):
        super(BaseContainersTestCase, self).setUp()

        self.cleanup = cleanup.CleanUp(self.barbicanclient)

        # Set up three secrets
        self.secret_ref_1, self.secret_1 = self._create_a_secret()
        self.secret_ref_2, self.secret_2 = self._create_a_secret()
        self.secret_ref_3, self.secret_3 = self._create_a_secret()

        self.secret_list = [self.secret_ref_1, self.secret_ref_2,
                            self.secret_ref_3]

        secrets_dict = {'secret_1': self.secret_1, 'secret_2': self.secret_2,
                        'secret_3': self.secret_3}

        create_container_defaults_data['secrets'] = secrets_dict

        create_container_rsa_data['public_key'] = self.secret_1
        create_container_rsa_data['private_key'] = self.secret_2
        create_container_rsa_data['private_key_passphrase'] = self.secret_3

    def tearDown(self):
        """Handles test cleanup.

        It should be noted that delete all secrets must be called before
        delete containers.
        """
        self.cleanup.delete_all_entities()
        super(BaseContainersTestCase, self).tearDown()

    def _create_a_secret(self):
        secret = self.barbicanclient.secrets.create(
            **create_secret_defaults_data)
        secret_ref = self.cleanup.add_entity(secret)

        return secret_ref, secret


@utils.parameterized_test_case
class GenericContainersTestCase(BaseContainersTestCase):

    @testcase.attr('positive')
    def test_create_container_defaults_none_secret_name(self):
        """Covers creating a container with None as a secret name."""
        container = self.barbicanclient.containers.create(
            **create_container_defaults_data)
        container.name = None

        container_ref = self.cleanup.add_entity(container)
        self.assertIsNotNone(container_ref)

    @testcase.attr('negative')
    def test_create_defaults_duplicate_secret_refs(self):
        """Covers creating a container with a duplicated secret ref."""
        secrets = {'secret_1': self.secret_1,
                   'secret_2': self.secret_1,
                   'secret_3': self.secret_1}

        create_container_defaults_data['secrets'] = secrets
        container = self.barbicanclient.containers.create(
            **create_container_defaults_data)

        e = self.assertRaises(
            exceptions.HTTPClientError,
            container.store
        )

        self.assertEqual(400, e.status_code)

    @testcase.attr('negative')
    def test_get_non_existent_container(self):
        """A get on a container that does not exist.

        This should return a container incorrectly specified error since
        the container does not have a correctly formatted UUID
        """
        base_url = self.barbicanclient.containers._api.endpoint_override
        url = base_url + '/containers/notauuid'
        e = self.assertRaises(ValueError, self.barbicanclient.containers.get,
                              url)

        self.assertEqual('Container incorrectly specified.', str(e))

    @testcase.attr('negative')
    def test_get_non_existent_container_valid_uuid(self):
        """A get on a container that does not exist with valid UUID

        This should return a 404.
        """
        base_url = self.barbicanclient.containers._api.endpoint_override
        uuid = 'de305d54-75b4-431b-cccc-eb6b9e546013'
        url = base_url + '/containers/' + uuid

        e = self.assertRaises(
            exceptions.HTTPClientError,
            self.barbicanclient.containers.get,
            url
        )

        self.assertEqual(404, e.status_code)

    @testcase.attr('negative')
    def test_delete_non_existent_container(self):
        """A delete on a container that does not exist.

        This should return a container incorrectly specified error since
        the container does not have a correctly formatted UUID
        """
        base_url = self.barbicanclient.containers._api.endpoint_override
        url = base_url + '/containers/notauuid'
        e = self.assertRaises(ValueError, self.barbicanclient.containers.get,
                              url)

        self.assertEqual('Container incorrectly specified.', str(e))

    @testcase.attr('negative')
    def test_delete_non_existent_container_valid_uuid(self):
        """A delete on a container that does not exist with valid UUID

        This should return a 404.
        """
        uuid = 'de305d54-75b4-431b-cccc-eb6b9e546013'
        base_url = self.barbicanclient.containers._api.endpoint_override
        url = base_url + '/containers/' + uuid

        e = self.assertRaises(
            exceptions.HTTPClientError,
            self.barbicanclient.containers.get,
            url
        )

        self.assertEqual(404, e.status_code)

    @utils.parameterized_dataset({'0': [0], '1': [1], '50': [50]})
    @testcase.attr('positive')
    def test_create_container_defaults_size(self, num_secrets):
        """Covers creating containers of various sizes."""
        secrets = {}
        for i in range(0, num_secrets):
            secret_ref, secret = self._create_a_secret()
            secrets['other_secret{0}'.format(i)] = secret

        create_container_defaults_data['secrets'] = secrets
        container = self.barbicanclient.containers.create(
            **create_container_defaults_data)

        container_ref = self.cleanup.add_entity(container)
        self.assertIsNotNone(container_ref)

    @utils.parameterized_dataset(accepted_str_values)
    @testcase.attr('positive')
    def test_create_container_defaults_name(self, name):
        """Covers creating generic containers with various names."""
        container = self.barbicanclient.containers.create(
            **create_container_defaults_data)
        container.name = name

        container_ref = self.cleanup.add_entity(container)
        self.assertIsNotNone(container_ref)

    @utils.parameterized_dataset(accepted_str_values)
    @testcase.attr('positive')
    def test_create_container_defaults_secret_name(self, name=None):
        """Covers creating containers with various secret ref names."""
        secrets = {name: self.secret_1}

        create_container_defaults_data['secrets'] = secrets
        container = self.barbicanclient.containers.create(
            **create_container_defaults_data)

        container_ref = self.cleanup.add_entity(container)
        self.assertIsNotNone(container_ref)

        container_resp = self.barbicanclient.containers.get(container_ref)
        self.assertIsNotNone(container_resp.secret_refs.get(name))

    @testcase.attr('positive')
    def test_container_read_with_acls(self):
        """Access default ACL settings data on recently created container.

        By default, 'read' ACL settings are there for a container.
         """
        test_model = self.barbicanclient.containers.create(
            **create_container_defaults_data)

        container_ref = self.cleanup.add_entity(test_model)
        self.assertIsNotNone(container_ref)

        container_entity = self.barbicanclient.containers.get(container_ref)
        self.assertIsNotNone(container_entity.acls)
        self.assertIsNotNone(container_entity.acls.read)
        self.assertEqual([], container_entity.acls.read.users)

    @utils.parameterized_dataset({
        'remove_one': [[{'name': 'ab', 'URL': 'http://c.d/e/1'},
                        {'name': 'ab', 'URL': 'http://c.d/e/2'}],
                       [{'name': 'ab', 'URL': 'http://c.d/e/1'}]],
        'remove_all': [[{'name': 'ab', 'URL': 'http://c.d/e/1'},
                        {'name': 'ab', 'URL': 'http://c.d/e/2'}],
                       [{'name': 'ab', 'URL': 'http://c.d/e/1'},
                        {'name': 'ab', 'URL': 'http://c.d/e/2'}]]
    })
    @testcase.attr('positive')
    def test_container_create_and_registering_removing_consumers(
            self,
            register_consumers,
            remove_consumers):

        new_container = self.barbicanclient.containers.create(
            **create_container_defaults_data)

        container_ref = self.cleanup.add_entity(new_container)
        self.assertIsNotNone(container_ref)

        for consumer in register_consumers:
            container = self.barbicanclient.containers.register_consumer(
                container_ref, consumer['name'], consumer['URL'])
            self.assertEqual(container_ref, container.container_ref)
        self.assertCountEqual(register_consumers, container.consumers)

        for consumer in remove_consumers:
            self.barbicanclient.containers.remove_consumer(
                container_ref, consumer['name'], consumer['URL'])

        container = self.barbicanclient.containers.get(container_ref)

        removed_urls = set([v['URL'] for v in remove_consumers])
        remaining_consumers = [v for v in register_consumers
                               if v['URL'] not in removed_urls]
        self.assertCountEqual(remaining_consumers, container.consumers)


@utils.parameterized_test_case
class RSAContainersTestCase(BaseContainersTestCase):
    @testcase.attr('positive')
    def test_create_containers_rsa_no_passphrase(self):
        """Covers creating an rsa container without a passphrase."""
        create_container_rsa_data['private_key_passphrase'] = None
        container = self.barbicanclient.containers.create_rsa(
            **create_container_rsa_data)

        container_ref = self.cleanup.add_entity(container)
        self.assertIsNotNone(container_ref)

        container_resp = self.barbicanclient.containers.get(container_ref)
        self.assertIsNone(container_resp.private_key_passphrase)
        self.assertEqual(2, len(container_resp.secrets))

    @utils.parameterized_dataset(accepted_str_values)
    @testcase.attr('positive')
    def test_create_container_rsa_name(self, name):
        """Covers creating rsa containers with various names."""
        container = self.barbicanclient.containers.create_rsa(
            **create_container_rsa_data)
        container.name = name

        container_ref = self.cleanup.add_entity(container)
        self.assertIsNotNone(container_ref)

        container_resp = self.barbicanclient.containers.get(container_ref)
        self.assertEqual(name, container_resp.name)

    @testcase.attr('negative')
    def test_create_rsa_invalid_key_names(self):
        """Covers creating an RSA container with incorrect names."""
        incorrect_names_rsa_container = {
            "name": "bad_container",
            "secret1": self.secret_ref_1,
            "secret2": self.secret_ref_2,
            "secret3": self.secret_ref_3
        }

        e = self.assertRaises(TypeError,
                              self.barbicanclient.containers.create_rsa,
                              **incorrect_names_rsa_container)

        self.assertIn('got an unexpected keyword argument', str(e))

    @testcase.attr('negative')
    def test_create_rsa_no_public_key(self):
        """Creating an rsa container without a public key should fail.

        RSA containers must have at least a public key and private key.
        """
        no_public_key_rsa_container = {"name": "no_pub_key",
                                       "private_key": self.secret_1,
                                       "private_key_passphrase": self.secret_2,
                                       }

        container = self.barbicanclient.containers.create_rsa(
            **no_public_key_rsa_container)

        e = self.assertRaises(
            exceptions.HTTPClientError,
            container.store
        )

        self.assertEqual(400, e.status_code)

    @testcase.attr('negative')
    def test_create_rsa_no_private_key(self):
        """Creating an rsa container without a private key should fail.

        RSA containers must have at least a public key and private key.
        """
        no_private_key_rsa_container = {
            "name": "no_pub_key",
            "public_key": self.secret_1,
            "private_key_passphrase": self.secret_2}

        container = self.barbicanclient.containers.create_rsa(
            **no_private_key_rsa_container)

        e = self.assertRaises(
            exceptions.HTTPClientError,
            container.store
        )

        self.assertEqual(400, e.status_code)



# Copyright (c) 2013 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Base utilities to build API operation managers.
"""
import logging
import uuid


LOG = logging.getLogger(__name__)


def filter_null_keys(dictionary):
    return dict(((k, v) for k, v in dictionary.items() if v is not None))


def censored_copy(data_dict, censor_keys):
    '''Returns redacted dict copy for censored keys'''
    if censor_keys is None:
        censor_keys = []
    return {k: v if k not in censor_keys else '<redacted>' for k, v in
            data_dict.items()}


def validate_ref_and_return_uuid(ref, entity):
    """Verifies that there is a real uuid (possibly at the end of a uri)

    :return: The uuid.UUID object
    :raises ValueError: If it cannot correctly parse the uuid in the ref.
    """
    try:
        # This works for a ref *or* a UUID, since we just pick the last piece
        ref_pieces = ref.rstrip('/').rsplit('/', 1)
        return uuid.UUID(ref_pieces[-1])
    except Exception:
        raise ValueError('{0} incorrectly specified.'.format(entity))


def calculate_uuid_ref(ref, entity):
    entity_uuid = validate_ref_and_return_uuid(
        ref, entity.capitalize().rstrip('s'))
    entity_ref = "{entity}/{uuid}".format(entity=entity, uuid=entity_uuid)
    LOG.info("Calculated %s uuid ref: %s", entity.capitalize(), entity_ref)
    return entity_ref


class ImmutableException(Exception):
    def __init__(self, attribute=None):
        message = "This object is immutable!"
        super(ImmutableException, self).__init__(message)


class BaseEntityManager(object):
    def __init__(self, api, entity):
        self._api = api
        self._entity = entity

    def total(self):
        """Returns the total number of entities stored in Barbican."""
        params = {'limit': 0, 'offset': 0}
        resp = self._api.get(self._entity, params=params)

        return resp['total']



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from functionaltests.cli.v1.behaviors import base_behaviors


class ContainerBehaviors(base_behaviors.BaseBehaviors):

    def __init__(self):
        super(ContainerBehaviors, self).__init__()
        self.LOG = logging.getLogger(type(self).__name__)
        self.container_hrefs_to_delete = []

    def delete_container(self, container_href):
        """Delete a container

        :param container_href the href to the container to delete
        """
        argv = ['secret', 'container', 'delete']
        self.add_auth_and_endpoint(argv)
        argv.extend([container_href])

        stdout, stderr = self.issue_barbican_command(argv)

        self.container_hrefs_to_delete.remove(container_href)

    def create_container(self, secret_hrefs=[]):
        """Create a container

        :param secret_hrefs A list of existing secrets

        :return: the href to the newly created container
        """
        argv = ['secret', 'container', 'create']
        self.add_auth_and_endpoint(argv)
        for secret_href in secret_hrefs:
            argv.extend(['--secret', secret_href])

        stdout, stderr = self.issue_barbican_command(argv)

        container_data = self._prettytable_to_dict(stdout)

        container_href = container_data['Container href']
        self.container_hrefs_to_delete.append(container_href)
        return container_href

    def get_container(self, container_href):
        """Get a container

        :param: the href to a container
        :return: dict of container values, or an empty dict if the container
        is not found.
        """
        argv = ['secret', 'container', 'get']
        self.add_auth_and_endpoint(argv)
        argv.extend([container_href])

        stdout, stderr = self.issue_barbican_command(argv)

        if '4xx Client error: Not Found' in stderr:
            return {}

        container_data = self._prettytable_to_dict(stdout)
        return container_data

    def list_containers(self):
        """List containers

        :return: a list of containers
        """
        argv = ['secret', 'container', 'list']

        self.add_auth_and_endpoint(argv)
        stdout, stderr = self.issue_barbican_command(argv)
        container_list = self._prettytable_to_list(stdout)
        return container_list

    def delete_all_created_containers(self):
        """Delete all containers that we created"""
        # Create a copy of the list -- otherwise delete_container will remove
        # items from the list as we are iterating over it
        containers_to_delete = list(self.container_hrefs_to_delete)
        for href in containers_to_delete:
            self.delete_container(href)



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from functionaltests.client import base
from functionaltests.common import cleanup
from functionaltests.common import keys
from functionaltests import utils
from testtools import testcase

secret_create_defaults_data = {
    "name": "AES key",
    "expiration": "2030-02-28T19:14:44.180394",
    "algorithm": "aes",
    "bit_length": 256,
    "mode": "cbc",
    "payload": b"gF6+lLoF3ohA9aPRpt+6bQ=="
}

secret_create_nones_data = {
    "name": None,
    "expiration": None,
    "algorithm": None,
    "bit_length": None,
    "mode": None,
    "payload": b"gF6+lLoF3ohA9aPRpt+6bQ==",
    "payload_content_type": "application/octet-stream",
    "payload_content_encoding": "base64",
}


@utils.parameterized_test_case
class SecretsTestCase(base.TestCase):

    def setUp(self):
        super(SecretsTestCase, self).setUp()
        self.cleanup = cleanup.CleanUp(self.barbicanclient)

    def tearDown(self):
        self.cleanup.delete_all_entities()
        super(SecretsTestCase, self).tearDown()

    @testcase.attr('positive')
    def test_create_secret_defaults(self):
        """Creates a secret with default values"""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

    @testcase.attr('positive')
    @utils.parameterized_dataset({
        'symmetric': ['symmetric',
                      'aes',
                      128,
                      (b'\x00\x01\x02\x03\x04\x05\x06\x07'
                       b'\x00\x01\x02\x03\x04\x05\x06\x07')],
        'private': ['private',
                    'rsa',
                    2048,
                    keys.get_private_key_pem()],
        'public': ['public',
                   'rsa',
                   2048,
                   keys.get_public_key_pem()],
        'certificate': ['certificate',
                        'rsa',
                        2048,
                        keys.get_certificate_pem()],
        'opaque': ['opaque',
                   None,
                   None,
                   (b'\x00\x01\x02\x03\x04\x05\x06\x07')],
        'passphrase': ['passphrase',
                       None,
                       None,
                       keys.get_passphrase_txt()],
    })
    def test_create_secret_with_type(self, secret_type, algorithm, bit_length,
                                     secret):
        """Creates a secret with default values"""
        secret_data = secret_create_defaults_data
        secret_data['secret_type'] = secret_type
        secret_data['algorithm'] = algorithm
        secret_data['bit_length'] = bit_length
        # payload should not be encoded.
        secret_data['payload'] = secret
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

    @testcase.attr('positive')
    def test_secret_create_defaults_no_expiration(self):
        """Covers creating a secret without an expiration."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.expiration = None

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

    @testcase.attr('positive')
    def test_secret_delete_defaults(self):
        """Covers deleting a secret."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)

        secret_ref = secret.store()

        del_response = self.barbicanclient.secrets.delete(secret_ref)
        self.assertIsNone(del_response)

    @testcase.attr('positive')
    def test_secret_delete_minimal_secret_w_no_metadata(self):
        """Covers deleting a secret with nones data."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_nones_data)

        secret_ref = secret.store()
        self.assertIsNotNone(secret_ref)

        del_resp = self.barbicanclient.secrets.delete(secret_ref)
        self.assertIsNone(del_resp)

    @testcase.attr('positive')
    def test_secret_get_defaults_payload(self):
        """Covers getting a secret's payload data."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret_ref = self.cleanup.add_entity(secret)

        secret_resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertEqual(secret.payload, secret_resp.payload)

    @testcase.attr('positive')
    def test_secrets_get_defaults_multiple_secrets(self):
        """Covers getting a list of secrets.

        Creates 11 secrets then returns a list of 5 secrets
        """
        limit = 5
        offset = 5
        total = 10

        for i in range(0, total + 1):
            secret = self.barbicanclient.secrets.create(
                **secret_create_defaults_data)
            self.cleanup.add_entity(secret)

        secret_list = self.barbicanclient.secrets.list(limit=limit,
                                                       offset=offset)

        self.assertEqual(limit, len(secret_list))



"""
Copyright 2022 Red Hat Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from unittest import mock

from keystoneauth1 import identity
from keystoneauth1 import session


_DEFAULT_ENDPOINT = "http://192.168.1.23/key-manager/"

STABLE_RESPONSE = {
    'version': {
        'id': 'v1',
        'status': 'stable',
        'links': [
            {
                'rel': 'self',
                'href': 'http://192.168.1.23/key-manager/v1/'
            }, {
                'rel': 'describedby',
                'type': 'text/html',
                'href': 'https://docs.openstack.org/'
            }],
        'media-types': [{
            'base': 'application/json',
            'type': 'application/vnd.openstack.key-manager-v1+json'
        }]
    }
}


def get_custom_current_response(min_version="1.0", max_version="1.1"):
    return {
        'version': {
            'id': 'v1',
            'status': 'CURRENT',
            'min_version': min_version,
            'max_version': max_version,
            'links': [
                {
                    'rel': 'self',
                    'href': 'http://192.168.1.23/key-manager/v1/'
                }, {
                    'rel': 'describedby',
                    'type': 'text/html',
                    'href': 'https://docs.openstack.org/'
                }
            ]
        }
    }


def mock_microversion_response(response=STABLE_RESPONSE):
    response_mock = mock.MagicMock()
    response_mock.json.return_value = response
    return response_mock


def get_version_endpoint(endpoint=None):
    return "{}/v1/".format(endpoint or _DEFAULT_ENDPOINT)


def mock_session():
    auth = identity.Password(
        auth_url="http://localhost/identity/v3",
        username="username",
        password="password",
        project_name="project_name",
        default_domain_id='default')
    sess = session.Session(auth=auth)
    return sess


def mock_session_get_endpoint(sess, endpoint_response):
    sess.get_endpoint = mock.MagicMock()
    sess.get_endpoint.return_value = endpoint_response


def mock_session_get(sess, get_response):
    response_mock = mock.MagicMock()
    response_mock.json.return_value = get_response

    sess.get = mock.MagicMock()
    sess.get.return_value = response_mock


def mock_session_with_get_and_get_endpoint(endpoint_response, get_response):
    sess = mock_session()
    mock_session_get(get_response)
    mock_session_get_endpoint(endpoint_response)

    return sess


def get_server_supported_versions(min_version, max_version):
    if min_version and max_version:
        return get_custom_current_response(min_version, max_version)
    return STABLE_RESPONSE


def mock_get_secret_for_client(client, consumers=[]):
    api_get_return = {
        'created': '2022-11-25T15:17:56',
        'updated': '2022-11-25T15:17:56',
        'status': 'ACTIVE',
        'name': 'Dummy secret',
        'secret_type': 'opaque',
        'expiration': None,
        'algorithm': None,
        'bit_length': None,
        'mode': None,
        'creator_id': '8ddfdbc4d92440369569af0589a20fa4',
        'consumers': consumers or [],
        'content_types': {'default': 'text/plain'},
        'secret_ref': 'http://192.168.1.23/key-manager/v1/'
                      'secrets/d46cfe10-c8ba-452f-a82f-a06834e45604'
    }
    client.client.get = mock.MagicMock()
    client.client.get.return_value = api_get_return


def mock_delete_secret_for_responses(responses, entity_href):
    responses.delete(entity_href, status_code=204)



# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from functionaltests.cli.base import CmdLineTestCase
from functionaltests.cli.v1.behaviors.consumer_behaviors import (
    ConsumerBehaviors)
from functionaltests.cli.v1.behaviors.secret_behaviors import SecretBehaviors
from functionaltests import utils
from testtools import testcase


@utils.parameterized_test_case
class ConsumerTestCase(CmdLineTestCase):

    def setUp(self):
        super(ConsumerTestCase, self).setUp()
        self.consumer_behaviors = ConsumerBehaviors()
        self.secret_behaviors = SecretBehaviors()

    def tearDown(self):
        super(ConsumerTestCase, self).tearDown()
        self.secret_behaviors.delete_all_created_secrets()

    def _create_secret(self):
        secret_href = self.secret_behaviors.store_secret()
        secret = self.secret_behaviors.get_secret(secret_href)
        return secret['Secret href']

    def _create_secret_with_consumer(self, consumer):
        secret_href = self._create_secret()
        self.consumer_behaviors.register_consumer(
            secret_href, consumer["service"], consumer["resource_type"],
            consumer["resource_id"])
        return secret_href

    def _register_consumer(self, secret_href, consumer):
        self.consumer_behaviors.register_consumer(
            secret_href, consumer["service"], consumer["resource_type"],
            consumer["resource_id"])

    @testcase.attr('positive')
    def test_register_consumer_on_empty_secret(self):
        consumer = {
            'service': 'service', 'resource_type': 'type',
            'resource_id': 'id', 'created': 'created'
        }
        secret_href = self._create_secret_with_consumer(consumer)

        secret = self.consumer_behaviors.list_consumers(secret_href)
        # Because "created" is non-deterministic, we need to assign
        # its value before running the loop below.
        secret[0]['Created'] = consumer['created']
        # The CLI's output is slighted different in terms of headers.
        # So, we have to rename their keys to the consumer dictionary's keys.
        for (k1, v1), (k2, v2) in zip(list(secret[0].items()),
                                      consumer.items()):
            secret[0][k2] = secret[0].pop(k1)
        self.assertDictEqual(consumer, secret[0])

    @testcase.attr('positive')
    def test_register_duplicated_service_name_consumer(self):
        consumer = {
            'service': 'service', 'resource_type': 'type', 'resource_id': 'id'
        }
        secret_href = self._create_secret_with_consumer(consumer)
        second_consumer = {
            'service': 'service', 'resource_type': 'type2',
            'resource_id': 'id2'
        }
        self._register_consumer(secret_href, second_consumer)
        self._register_consumer(secret_href, second_consumer)
        consumers_list = self.consumer_behaviors.list_consumers(secret_href)
        self.assertEqual(2, len(consumers_list))

    @testcase.attr('positive')
    def test_register_duplicated_resource_type_consumer(self):
        consumer = {
            'service': 'service', 'resource_type': 'type',
            'resource_id': 'id'
        }
        secret_href = self._create_secret_with_consumer(consumer)
        second_consumer = {
            'service': 'service2', 'resource_type': 'type',
            'resource_id': 'id2'
        }
        self._register_consumer(secret_href, second_consumer)
        consumers_list = self.consumer_behaviors.list_consumers(secret_href)
        self.assertEqual(2, len(consumers_list))

    @testcase.attr('positive')
    def test_register_duplicated_resource_id_consumer(self):
        consumer = {
            'service': 'service', 'resource_type': 'type', 'resource_id': 'id'
        }
        secret_href = self._create_secret_with_consumer(consumer)
        second_consumer = {
            'service': 'service2', 'resource_type': 'type2',
            'resource_id': 'id'
        }
        self._register_consumer(secret_href, second_consumer)
        consumers_list = self.consumer_behaviors.list_consumers(secret_href)
        self.assertEqual(2, len(consumers_list))

    @testcase.attr('positive')
    def test_remove_consumer(self):
        consumer = {
            'service': 'service', 'resource_type': 'type',
            'resource_id': 'id'
        }
        secret_href = self._create_secret_with_consumer(consumer)

        self.consumer_behaviors.remove_consumer(
            secret_href, consumer["service"], consumer["resource_type"],
            consumer["resource_id"])

        consumers = self.consumer_behaviors.list_consumers(secret_href)
        self.assertEqual(0, len(consumers))

    @testcase.attr('positive')
    def test_list_consumer_secret_with_multiple_consumers(self):
        first_consumer = {
            'service': 'service1', 'resource_type': 'type1',
            'resource_id': 'id1'}
        secret_href = self._create_secret_with_consumer(first_consumer)

        second_consumer = {
            'service': 'service2', 'resource_type': 'type2',
            'resource_id': 'id2'}
        self._register_consumer(secret_href, second_consumer)

        consumers = self.consumer_behaviors.list_consumers(secret_href)
        self.assertEqual(2, len(consumers))



# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from functionaltests.cli.v1.behaviors import base_behaviors


class ConsumerBehaviors(base_behaviors.BaseBehaviors):

    def __init__(self):
        super(ConsumerBehaviors, self).__init__()
        self.LOG = logging.getLogger(type(self).__name__)

    def register_consumer(self, secret_href, service, resource_type,
                          resource_id):
        argv = ['secret', 'consumer', 'create']
        self.add_auth_and_endpoint(argv)

        argv.extend(['--service-type-name', service])
        argv.extend(['--resource-type', resource_type])
        argv.extend(['--resource-id', resource_id])
        argv.extend([secret_href])

        stdout, stderr = self.issue_barbican_command(argv)

    def remove_consumer(self, secret_href, service, resource_type,
                        resource_id):
        argv = ['secret', 'consumer', 'delete']
        self.add_auth_and_endpoint(argv)

        argv.extend(['--service-type-name', service])
        argv.extend(['--resource-type', resource_type])
        argv.extend(['--resource-id', resource_id])
        argv.extend([secret_href])

        stdout, stderr = self.issue_barbican_command(argv)

    def list_consumers(self, secret_href):
        argv = ['secret', 'consumer', 'list']
        self.add_auth_and_endpoint(argv)

        argv.extend([secret_href])

        stdout, stderr = self.issue_barbican_command(argv)

        if len(stderr) > 0 or stdout == '\n':
            return []
        else:
            consumers = self._prettytable_to_list(stdout)
            return consumers



# Copyright (c) 2013 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from oslo_utils import timeutils

from barbicanclient.tests import test_client
from barbicanclient.v1 import cas


class CAData(object):
    def __init__(self, description='Test CA description'):
        self.name = 'Test CA'
        self.description = description
        self.plugin_name = 'Test CA Plugin'
        self.plugin_ca_id = 'plugin_uuid'

        now = timeutils.utcnow()
        self.expiration = str(now)
        self.created = str(now)

        self.meta = []
        self.meta.append({'name': self.name})
        if self.description:
            self.meta.append({'description': self.description})

        self.ca_dict = {'meta': self.meta,
                        'status': 'ACTIVE',
                        'plugin_name': self.plugin_name,
                        'plugin_ca_id': self.plugin_ca_id,
                        'created': self.created}

    def get_dict(self, ca_ref=None):
        ca = self.ca_dict
        if ca_ref:
            ca['ca_ref'] = ca_ref
        return ca


class WhenTestingCAs(test_client.BaseEntityResource):

    def setUp(self):
        self._setUp('cas')

        self.ca = CAData()
        self.manager = self.client.cas

    def test_should_get_lazy(self, ca_ref=None):
        ca_ref = ca_ref or self.entity_href

        data = self.ca.get_dict(ca_ref)
        m = self.responses.get(self.entity_href, json=data)

        ca = self.manager.get(ca_ref=ca_ref)
        self.assertIsInstance(ca, cas.CA)
        self.assertEqual(ca_ref, ca._ca_ref)

        # Verify GET wasn't called yet
        self.assertFalse(m.called)

        # Check an attribute to trigger lazy-load
        self.assertEqual(self.ca.plugin_ca_id, ca.plugin_ca_id)

        # Verify the correct URL was used to make the GET call
        self.assertEqual(self.entity_href, m.last_request.url)

    def test_should_get_lazy_using_stripped_uuid(self):
        bad_href = "http://badsite.com/" + self.entity_id
        self.test_should_get_lazy(bad_href)

    def test_should_get_lazy_using_only_uuid(self):
        self.test_should_get_lazy(self.entity_id)

    def test_should_get_lazy_in_meta(self):
        data = self.ca.get_dict(self.entity_href)
        m = self.responses.get(self.entity_href, json=data)

        ca = self.manager.get(ca_ref=self.entity_href)
        self.assertIsInstance(ca, cas.CA)
        self.assertEqual(self.entity_href, ca._ca_ref)

        # Verify GET wasn't called yet
        self.assertFalse(m.called)

        # Check an attribute in meta to trigger lazy-load
        self.assertEqual(self.ca.name, ca.name)

        # Verify the correct URL was used to make the GET call
        self.assertEqual(self.entity_href, m.last_request.url)

    def test_should_get_list(self):
        ca_resp = self.entity_href

        data = {"cas": [ca_resp for v in range(3)]}
        m = self.responses.get(self.entity_base, json=data)

        ca_list = self.manager.list(limit=10, offset=5)
        self.assertTrue(len(ca_list) == 3)
        self.assertIsInstance(ca_list[0], cas.CA)
        self.assertEqual(self.entity_href, ca_list[0].ca_ref)

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_base,
                         m.last_request.url.split('?')[0])

        # Verify that correct information was sent in the call.
        self.assertEqual(['10'], m.last_request.qs['limit'])
        self.assertEqual(['5'], m.last_request.qs['offset'])

    def test_should_fail_get_invalid_ca(self):
        self.assertRaises(ValueError, self.manager.get,
                          **{'ca_ref': '12345'})

    def test_should_get_ca_that_has_no_meta_description(self):
        self.ca = CAData(description=None)

        data = self.ca.get_dict(self.entity_href)
        m = self.responses.get(self.entity_href, json=data)

        ca = self.manager.get(ca_ref=self.entity_href)
        self.assertIsInstance(ca, cas.CA)
        self.assertEqual(self.entity_href, ca._ca_ref)

        # Verify GET wasn't called yet
        self.assertFalse(m.called)

        # Get description from CA, check it is None
        self.assertIsNone(self.ca.description)
        self.assertIsNone(ca.description)

        # Verify the correct URL was used to make the GET call
        self.assertEqual(self.entity_href, m.last_request.url)

    def test_get_formatted_data(self):
        c_entity = cas.CA(api=None,
                          expiration=self.ca.expiration,
                          plugin_name=self.ca.plugin_name,
                          created=self.ca.created)

        data = c_entity._get_formatted_data()

        self.assertEqual(self.ca.plugin_name, data[6])
        self.assertEqual(timeutils.parse_isotime(
                         self.ca.expiration).isoformat(),
                         data[8])



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
import datetime
import sys
import time

from oslo_utils import timeutils

from barbicanclient import exceptions
from functionaltests.client import base
from functionaltests.common import cleanup
from functionaltests.common import keys
from functionaltests import utils
from testtools import testcase


secret_create_defaults_data = {
    "name": "AES key",
    "expiration": "2030-02-28T19:14:44.180394",
    "algorithm": "aes",
    "bit_length": 256,
    "mode": "cbc",
    "payload": b"gF6+lLoF3ohA9aPRpt+6bQ=="
}

secret_create_nones_data = {
    "name": None,
    "expiration": None,
    "algorithm": None,
    "bit_length": None,
    "mode": None,
    "payload": b"gF6+lLoF3ohA9aPRpt+6bQ==",
    "payload_content_type": "application/octet-stream",
    "payload_content_encoding": "base64",
}

secret_create_emptystrings_data = {
    "name": '',
    "expiration": '',
    "algorithm": '',
    "bit_length": '',
    "mode": '',
    "payload": b'',
    "payload_content_type": '',
    "payload_content_encoding": '',
}


@utils.parameterized_test_case
class SecretsTestCase(base.TestCase):

    def setUp(self):
        super(SecretsTestCase, self).setUp()
        self.cleanup = cleanup.CleanUp(self.barbicanclient)

    def tearDown(self):
        self.cleanup.delete_all_entities()
        super(SecretsTestCase, self).tearDown()

    def _create_test_secret(self):
        """Helper module to create a secret withouth consumers"""
        new_secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)

        secret_ref = self.cleanup.add_entity(new_secret)
        self.assertIsNotNone(secret_ref)

        return secret_ref

    @testcase.attr('positive')
    def test_secret_create_defaults_check_content_types(self):
        """Check that set content-type attribute is retained in metadata."""
        secret_ref = self._create_test_secret()

        resp = self.barbicanclient.secrets.get(secret_ref)
        content_types = resp.content_types
        self.assertIsNotNone(content_types)
        self.assertIn('default', content_types)
        self.assertEqual('application/octet-stream', content_types['default'])

    @testcase.attr('positive')
    def test_secret_create_defaults_non_standard_algorithm(self):
        """Create a secret with a non standard algorithm.

         Currently the client will accept any string for the algorithm.
         """
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.algorithm = "not-an-algorithm"

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertEqual(secret.algorithm, resp.algorithm)

    @testcase.attr('positive')
    def test_secret_read_with_acls(self):
        """Access default ACL settings data on recently created secret.

        By default, 'read' ACL settings are there for a secret.
         """
        secret_ref = self._create_test_secret()

        secret_entity = self.barbicanclient.secrets.get(secret_ref)
        self.assertIsNotNone(secret_entity.acls)
        self.assertIsNotNone(secret_entity.acls.read)
        self.assertEqual([], secret_entity.acls.read.users)

    @testcase.attr('positive')
    def test_secret_create_defaults_non_standard_mode(self):
        """Create a secret with a non standard mode.

        Currently the client will accept any string for the mode.
        """
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.mode = 'not-a-mode'

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertEqual(secret.mode, resp.mode)

    @utils.parameterized_dataset({
        'remove_one': [[{'service': 'service_test1',
                         'resource_type': 'type_test1',
                         'resource_id': 'id_test1'},
                        {'service': 'service_test2',
                         'resource_type': 'type_test2',
                         'resource_id': 'id_test2'}],
                       [{'service': 'service_test1',
                         'resource_type': 'type_test1',
                         'resource_id': 'id_test1'}]],
        'remove_all': [[{'service': 'service_test1',
                         'resource_type': 'type_test1',
                         'resource_id': 'id_test1'},
                        {'service': 'service_test2',
                         'resource_type': 'type_test2',
                         'resource_id': 'id_test2'}],
                       [{'service': 'service_test1',
                         'resource_type': 'type_test1',
                         'resource_id': 'id_test1'},
                        {'service': 'service_test2',
                         'resource_type': 'type_test2',
                         'resource_id': 'id_test2'}]],
        'add_duplicate_remove_one': [[{'service': 'service_test1',
                                       'resource_type': 'type_test1',
                                       'resource_id': 'id_test1'},
                                      {'service': 'service_test1',
                                       'resource_type': 'type_test1',
                                       'resource_id': 'id_test1'},
                                      {'service': 'service_test2',
                                       'resource_type': 'type_test2',
                                       'resource_id': 'id_test2'}],
                                     [{'service': 'service_test1',
                                       'resource_type': 'type_test1',
                                       'resource_id': 'id_test1'}]]
    })
    @testcase.attr('positive')
    def test_secret_create_and_registering_removing_consumers(
            self,
            register_consumers,
            remove_consumers):
        """The following activities are carried:

        Create a secret, then register each consumer
        in the register_consumers list, then remove each consumer
        in the remove_consumers list.
        """
        secret_ref = self._create_test_secret()

        for consumer in register_consumers:
            secret = self.barbicanclient.secrets.register_consumer(
                secret_ref, **consumer)
            self.assertEqual(secret_ref, secret.secret_ref)

        # We expect that duplicate calls to register_consumers don't
        # create new consumers even though the API returns HTTP 200 OK
        deduplicated_consumers_count = len(set(
            [c['resource_id'] for c in register_consumers]))

        self.assertEqual(deduplicated_consumers_count,
                         len(secret.consumers))

        for consumer in remove_consumers:
            self.barbicanclient.secrets.remove_consumer(
                secret_ref, **consumer)

        secret = self.barbicanclient.secrets.get(secret_ref)

        removed_ids = set([v['resource_id'] for v in remove_consumers])
        remaining_consumers = [v for v in register_consumers
                               if v['resource_id'] not in removed_ids]
        self.assertCountEqual(remaining_consumers, secret.consumers)

    @utils.parameterized_dataset({
        'no_args': [[{}]],
        'one_arg_1': [[{'service': 'service1'}]],
        'one_arg_2': [[{'resource_type': 'type1'}]],
        'one_arg_3': [[{'resource_id': 'id1'}]],
        'two_args_1': [[{'service': 'service1',
                         'resource_type': 'type1'}]],
        'two_args_2': [[{'service': 'service1',
                         'resource_id': 'id1'}]],
        'two_args_3': [[{'resource_type': 'type1',
                         'resource_id': 'id'}]]
    })
    @testcase.attr('negative')
    def test_consumer_register_missing_positional_arguments(
            self,
            register_consumers):
        """Missing Positional Arguments - Registration

        Tries to register a secret consumer without
        providing all of the required positional arguments
        (service, resource_type, resource_id).
        """
        secret_ref = self._create_test_secret()

        for consumer in register_consumers:
            e = self.assertRaises(
                TypeError,
                self.barbicanclient.secrets.register_consumer,
                secret_ref, **consumer
            )
        self.assertIn('register_consumer() missing', str(e))

    @utils.parameterized_dataset({
        'no_args': [[{}]],
        'one_arg_1': [[{'service': 'service1'}]],
        'one_arg_2': [[{'resource_type': 'type1'}]],
        'one_arg_3': [[{'resource_id': 'id1'}]],
        'two_args_1': [[{'service': 'service1',
                         'resource_type': 'type1'}]],
        'two_args_2': [[{'service': 'service1',
                         'resource_id': 'id1'}]],
        'two_args_3': [[{'resource_type': 'type1',
                         'resource_id': 'id'}]]
    })
    @testcase.attr('negative')
    def test_consumer_remove_missing_positional_arguments(
            self,
            register_consumers):
        """Missing Positional Arguments - Removal

        Tries to remove a secret consumer without
        providing all of the required positional arguments
        (service, resource_type, resource_id).
        """
        secret_ref = self._create_test_secret()

        secret = self.barbicanclient.secrets.register_consumer(
            secret_ref,
            service="service1",
            resource_type="type1",
            resource_id="id1"
        )
        self.assertEqual(secret_ref, secret.secret_ref)
        for consumer in register_consumers:
            e = self.assertRaises(
                TypeError,
                self.barbicanclient.secrets.remove_consumer,
                secret_ref, **consumer
            )
        self.assertIn('remove_consumer() missing', str(e))

    @testcase.attr('positive')
    def test_secret_delete_without_consumers_no_force(self):
        secret_ref = self._create_test_secret()

        self.barbicanclient.secrets.delete(secret_ref, force=False)
        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertRaises(exceptions.HTTPClientError, getattr, resp, "name")
        self.cleanup.delete_entity(secret_ref)

    @testcase.attr('positive')
    def test_secret_delete_without_consumers_with_force(self):
        secret_ref = self._create_test_secret()

        self.barbicanclient.secrets.delete(secret_ref, force=True)
        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertRaises(exceptions.HTTPClientError, getattr, resp, "name")
        self.cleanup.delete_entity(secret_ref)

    @testcase.attr('negative')
    def test_secret_delete_with_consumers_no_force_version_1_1(self):
        """Deleting a secret with consumers using microversion 1.1

        Tries to delete a secret with consumers, but
        without providing the 'force' parameter.
        """
        self.barbicanclient._set_microversion('1.1')
        secret_ref = self._create_test_secret()

        secret = self.barbicanclient.secrets.register_consumer(
            secret_ref,
            service="service1",
            resource_type="type1",
            resource_id="id1"
        )
        self.assertEqual(secret_ref, secret.secret_ref)

        self.assertRaises(exceptions.SecretHasConsumers,
                          self.barbicanclient.secrets.delete,
                          secret.secret_ref)

    @testcase.attr('positive')
    def test_secret_delete_with_consumers_with_force_version_1_1(self):
        """Deleting a secret with consumers using microversion 1.1

        Tries to delete a secret with consumers,
        making the 'force' parameter equals True.
        """
        self.barbicanclient._set_microversion('1.1')
        secret_ref = self._create_test_secret()

        secret = self.barbicanclient.secrets.register_consumer(
            secret_ref,
            service="service1",
            resource_type="type1",
            resource_id="id1"
        )
        self.assertEqual(secret_ref, secret.secret_ref)

        self.barbicanclient.secrets.delete(secret.secret_ref, True)
        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertRaises(exceptions.HTTPClientError, getattr, resp, "name")
        self.cleanup.delete_entity(secret_ref)

    @testcase.attr('negative')
    def test_secret_delete_with_consumers_no_force_version_1_2(self):
        """Deleting a secret with consumers using microversion 1.2

        Tries to delete a secret with consumers, but
        without providing the 'force' parameter.
        """
        self.barbicanclient._set_microversion('1.2')
        secret_ref = self._create_test_secret()

        secret = self.barbicanclient.secrets.register_consumer(
            secret_ref,
            service="service1",
            resource_type="type1",
            resource_id="id1"
        )
        self.assertEqual(secret_ref, secret.secret_ref)

        self.assertRaises(exceptions.SecretHasConsumers,
                          self.barbicanclient.secrets.delete,
                          secret.secret_ref)

    @testcase.attr('positive')
    def test_secret_delete_with_consumers_with_force_version_1_2(self):
        """Deleting a secret with consumers using microversion 1.2

        Tries to delete a secret with consumers,
        making the 'force' parameter equals True.
        """
        self.barbicanclient._set_microversion('1.2')
        secret_ref = self._create_test_secret()

        secret = self.barbicanclient.secrets.register_consumer(
            secret_ref,
            service="service1",
            resource_type="type1",
            resource_id="id1"
        )
        self.assertEqual(secret_ref, secret.secret_ref)

        self.barbicanclient.secrets.delete(secret.secret_ref, True)
        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertRaises(exceptions.HTTPClientError, getattr, resp, "name")
        self.cleanup.delete_entity(secret_ref)

    @testcase.attr('positive')
    def test_consumers_list_secret_without_consumers(self):
        """Lists consumers from a secret without consumers"""
        secret_ref = self._create_test_secret()

        consumers_list = self.barbicanclient.secrets.list_consumers(
            secret_ref)
        self.assertTrue(len(consumers_list) == 0)

        self.cleanup.delete_entity(secret_ref)
        self.barbicanclient.secrets.delete(secret_ref, True)

    @testcase.attr('positive')
    def test_consumers_list_secret_with_consumers(self):
        """Lists consumers from a secret with consumers"""
        secret_ref = self._create_test_secret()

        consumers = [{
            'service': 'service1',
            'resource_type': 'type1',
            'resource_id': 'id1'}, {
            'service': 'service2',
            'resource_type': 'type2',
            'resource_id': 'id2'}]

        for consumer in consumers:
            _ = self.barbicanclient.secrets.register_consumer(
                secret_ref,
                service=consumer['service'],
                resource_type=consumer['resource_type'],
                resource_id=consumer['resource_id']
            )

        consumers_list = self.barbicanclient.secrets.list_consumers(
            secret_ref)

        for elem in range(len(consumers)):
            self.assertTrue(
                consumers_list[elem].service ==
                consumers[elem]['service'])
            self.assertTrue(
                consumers_list[elem].resource_type ==
                consumers[elem]['resource_type'])
            self.assertTrue(
                consumers_list[elem].resource_id ==
                consumers[elem]['resource_id'])

        self.cleanup.delete_entity(secret_ref)
        self.barbicanclient.secrets.delete(secret_ref, True)

    @testcase.attr('negative')
    def test_consumers_list_secret_doesnt_exist(self):
        """Tries to list consumers from a non-existent secret"""
        e = self.assertRaises(exceptions.HTTPClientError,
                              self.barbicanclient.secrets.list_consumers,
                              '9999999f-f99f-49f9-9fff-f99f999ff9ff')

        self.assertIn("Secret not found", str(e))

    @testcase.attr('negative')
    def test_consumers_list_secret_invalid_uuid(self):
        """Tries to list consumers providing an invalid secret UUID"""
        e = self.assertRaises(exceptions.HTTPClientError,
                              self.barbicanclient.secrets.list_consumers,
                              '9999999f-ffff-ffff-9fff-f99f999ff9ff')

        self.assertIn("Provided secret id is invalid.", str(e))

    @testcase.attr('negative')
    def test_consumers_list_invalid_secret(self):
        """Tries to list consumers providing an invalid secret"""
        e = self.assertRaises(ValueError,
                              self.barbicanclient.secrets.list_consumers,
                              'abcde')

        self.assertIn("secret incorrectly specified.", str(e))

    @testcase.attr('negative')
    def test_secret_delete_doesnt_exist(self):
        """Deletes a non-existent secret.

        This delete uses a reference with an invalid UUID format
        """
        url = self.barbicanclient.secrets._api.endpoint_override + \
            '/secrets/notauuid'

        e = self.assertRaises(ValueError, self.barbicanclient.secrets.delete,
                              url)

        self.assertEqual('Secret incorrectly specified.', str(e))

    @testcase.attr('negative')
    def test_secret_delete_doesnt_exist_valid_uuid_format(self):
        """Deletes a non-existent secret.

        This delete has a valid UUID format but there is no secret
        associated with this UUID
        """
        uuid = 'de20ad54-85b4-421b-adb2-eb7b9e546013'
        url = self.barbicanclient.secrets._api.endpoint_override + \
            '/secrets/' + uuid

        e = self.assertRaises(
            exceptions.HTTPClientError,
            self.barbicanclient.secrets.delete,
            url
        )

        self.assertEqual(404, e.status_code)

    @testcase.attr('negative')
    def test_secret_get_secret_doesnt_exist(self):
        """GET an invalid secret ref.

        Will get value error secret incorrectly specified since "notauuid"
        is not a properly formatted uuid.
        """
        url = self.barbicanclient.secrets._api.endpoint_override + \
            '/secrets/notauuid'

        e = self.assertRaises(ValueError, self.barbicanclient.secrets.get,
                              url, 'text/plain')

        self.assertIn("Secret incorrectly specified", str(e))

    @testcase.attr('negative')
    def test_secret_create_defaults_expiration_passed(self):
        """Create a secret with an expiration that has already passed.

        Returns a 400.
        """
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.expiration = '2000-01-10T14:58:52.546795'

        e = self.assertRaises(
            exceptions.HTTPClientError,
            secret.store
        )
        self.assertEqual(400, e.status_code)

    @testcase.attr('negative')
    def test_secret_create_emptystrings(self):
        """Secret create with empty Strings for all attributes.

        Fails with a value error, Payload incorrectly specified.
        """
        secret = self.barbicanclient.secrets.create(
            **secret_create_emptystrings_data)

        self.assertRaises(
            exceptions.PayloadException,
            secret.store
        )

    @testcase.attr('negative')
    def test_secret_create_defaults_oversized_payload(self):
        """Create a secret with a payload that is larger than the allowed size.

        Should return a 413 if the secret size is greater than the
        maximum allowed size.
        """
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.payload = str(self.oversized_payload)

        e = self.assertRaises(
            exceptions.HTTPClientError,
            secret.store
        )
        self.assertEqual(413, e.status_code)

    @utils.parameterized_dataset({
        'alphanumeric': ['1f34ds'],
        'punctuation': ['~!@#$%^&*()_+`-={}[]|:;<>,.?'],
        'uuid': ['54262d9d-4bc7-4821-8df0-dc2ca8e112bb'],
        'len_255': [base.TestCase.max_sized_field],
        'empty': [''],
        'null': [None]
    })
    @testcase.attr('positive')
    def test_secret_create_defaults_valid_name(self, name):
        """Covers cases of creating secrets with valid names."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.name = name

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertEqual(secret.name, resp.name)

    @utils.parameterized_dataset({
        'int': [400]
    })
    @testcase.attr('negative')
    def test_secret_create_defaults_invalid_name(self, name):
        """Create secrets with various invalid names.

        Should return 400.
        """
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.name = name

        e = self.assertRaises(
            exceptions.HTTPClientError,
            secret.store
        )

        self.assertEqual(400, e.status_code)

    @utils.parameterized_dataset({
        'aes': ['aes']
    })
    @testcase.attr('positive')
    def test_secret_create_defaults_valid_algorithms(self, algorithm):
        """Creates secrets with various valid algorithms."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.algorithm = algorithm

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertEqual(secret.algorithm, resp.algorithm)

    @utils.parameterized_dataset({
        'int': [400]
    })
    @testcase.attr('negative')
    def test_secret_create_defaults_invalid_algorithms(self, algorithm):
        """Creates secrets with various invalid algorithms."""

        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.algorithm = algorithm

        # We are currently testing for exception with http_code
        # launchpad bug 1431514 will address the change to this functionality
        e = self.assertRaises(
            exceptions.HTTPClientError,
            secret.store
        )

        self.assertEqual(400, e.status_code)

    @utils.parameterized_dataset({
        'symmetric': ['symmetric',
                      'aes',
                      128,
                      (b'\x00\x01\x02\x03\x04\x05\x06\x07'
                       b'\x00\x01\x02\x03\x04\x05\x06\x07')],
        'private': ['private',
                    'rsa',
                    2048,
                    keys.get_private_key_pem()],
        'public': ['public',
                   'rsa',
                   2048,
                   keys.get_public_key_pem()],
        'certificate': ['certificate',
                        'rsa',
                        2048,
                        keys.get_certificate_pem()],
        'opaque': ['opaque',
                   None,
                   None,
                   (b'\x00\x01\x02\x03\x04\x05\x06\x07')],
        'passphrase': ['passphrase',
                       None,
                       None,
                       keys.get_passphrase_txt()],
    })
    @testcase.attr('positive')
    def test_secret_create_defaults_valid_secret_type(
            self, secret_type, algorithm, bit_length, payload):
        """Covers cases of creating secrets with valid secret types."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.secret_type = secret_type
        secret.algorithm = algorithm
        secret.bit_length = bit_length
        # payload should not be encoded.
        secret.payload = payload

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertEqual(secret_type, resp.secret_type)

    @testcase.attr('negative')
    def test_secret_create_defaults_invalid_secret_type(self):
        """Covers cases of creating secrets with invalid secret types."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.secret_type = 'not a valid secret type'

        e = self.assertRaises(
            exceptions.HTTPClientError,
            secret.store
        )

        self.assertEqual(400, e.status_code)

    @utils.parameterized_dataset({
        '512': [512],
        'sixteen': [16],
        'fifteen': [15],
        'eight': [8],
        'seven': [7],
        'one': [1],
        'none': [None]
    })
    @testcase.attr('positive')
    def test_secret_create_defaults_valid_bit_length(self, bit_length):
        """Covers cases of creating secrets with valid bit lengths."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.bit_length = bit_length

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertEqual(secret.bit_length, resp.bit_length)

    @utils.parameterized_dataset({
        'str_type': ['not-an-int'],
        'empty': [''],
        'blank': [' '],
        'negative_maxsize': [-sys.maxsize],
        'negative_one': [-1],
        'zero': [0]
    })
    @testcase.attr('negative')
    def test_secret_create_defaults_invalid_bit_length(self, bit_length):
        """Covers cases of creating secrets with invalid bit lengths."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.bit_length = bit_length

        e = self.assertRaises(
            exceptions.HTTPClientError,
            secret.store
        )

        self.assertEqual(400, e.status_code)

    @utils.parameterized_dataset({
        'cbc': ['cbc']
    })
    @testcase.attr('positive')
    def test_secret_create_defaults_valid_mode(self, mode):
        """Covers cases of creating secrets with valid modes."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.mode = mode

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertEqual(secret.mode, resp.mode)

    @utils.parameterized_dataset({
        'zero': [0],
        'oversized_string': [base.TestCase.oversized_field],
        'int': [400]
    })
    @testcase.attr('negative')
    def test_secret_create_defaults_invalid_mode(self, mode):
        """Covers cases of creating secrets with invalid modes."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.mode = mode

        e = self.assertRaises(
            exceptions.HTTPClientError,
            secret.store
        )

        self.assertEqual(400, e.status_code)

    @utils.parameterized_dataset({
        'text_content_type_none_encoding': {
            'payload_content_type': 'text/plain',
            'payload_content_encoding': None},

        'utf8_text_content_type_none_encoding': {
            'payload_content_type': 'text/plain; charset=utf-8',
            'payload_content_encoding': None},

        'no_space_utf8_text_content_type_none_encoding': {
            'payload_content_type': 'text/plain;charset=utf-8',
            'payload_content_encoding': None},

        'octet_content_type_base64_encoding': {
            'payload_content_type': 'application/octet-stream',
            'payload_content_encoding': 'base64'}
    })
    @testcase.attr('positive')
    def test_secret_create_deprecated_types_and_encoding(
            self,
            payload_content_type,
            payload_content_encoding):
        """Creates secrets with various content types and encodings."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.payload_content_type = payload_content_type
        secret.payload_content_encoding = payload_content_encoding

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        resp = self.barbicanclient.secrets.get(secret_ref)
        if secret.payload_content_encoding == 'base64':
            self.assertEqual(
                base64.b64decode(secret.payload),
                resp.payload
            )
        else:
            self.assertEqual(secret.payload, resp.payload.encode('utf-8'))

    @utils.parameterized_dataset({
        'large_string_content_type_and_encoding': {
            'payload_content_type': base.TestCase.oversized_field,
            'payload_content_encoding': base.TestCase.oversized_field},

        'int_content_type_and_encoding': {
            'payload_content_type': 123,
            'payload_content_encoding': 123},

        'text_content_type_none_content_encoding': {
            'payload_content_type': 'text/plain',
            'payload_content_encoding': ''},

        'text_no_subtype_content_type_none_content_encoding': {
            'payload_content_type': 'text',
            'payload_content_encoding': None},

        'text_slash_no_subtype_content_type_none_content_encoding': {
            'payload_content_type': 'text/',
            'payload_content_encoding': None},

        'text_content_type_empty_content_encoding': {
            'payload_content_type': 'text/plain',
            'payload_content_encoding': ' '},

        'text_content_type_spaces_content_encoding': {
            'payload_content_type': 'text/plain',
            'payload_content_encoding': ' '},

        'text_content_type_base64_content_encoding': {
            'payload_content_type': 'text/plain',
            'payload_content_encoding': 'base64'},

        'text_and_utf88_content_type_none_content_encoding': {
            'payload_content_type': 'text/plain; charset=utf-88',
            'payload_content_encoding': None},

        'invalid_content_type_base64_content_encoding': {
            'payload_content_type': 'invalid',
            'payload_content_encoding': 'base64'},

        'invalid_content_type_none_content_encoding': {
            'payload_content_type': 'invalid',
            'payload_content_encoding': None},

        'octet_content_type_invalid_content_encoding': {
            'payload_content_type': 'application/octet-stream',
            'payload_content_encoding': 'invalid'},

        'text_content_type_invalid_content_encoding': {
            'payload_content_type': 'text/plain',
            'payload_content_encoding': 'invalid'},

        'none_content_type_invalid_content_encoding': {
            'payload_content_type': None,
            'payload_content_encoding': 'invalid'},

        'none_content_type_base64_content_encoding': {
            'payload_content_type': None,
            'payload_content_encoding': 'base64'}
    })
    @testcase.attr('negative')
    def test_secret_create_defaults_invalid_types_and_encoding(self, **kwargs):
        """Creating secrets with invalid payload types and encodings."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.payload_content_encoding = kwargs[
            'payload_content_encoding']
        secret.payload_content_type = kwargs[
            'payload_content_type']

        e = self.assertRaises(
            exceptions.HTTPClientError,
            secret.store
        )

        self.assertEqual(400, e.status_code)

    @utils.parameterized_dataset({
        'max_payload_string': [base.TestCase.max_sized_payload]
    })
    @testcase.attr('positive')
    def test_secret_create_defaults_valid_payload(self, payload):
        """Create secrets with a various valid payloads."""

        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.payload = payload

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertEqual(secret.payload, resp.payload)

    @utils.parameterized_dataset({
        'list': [['boom']],
        'int': [123]
    })
    @testcase.attr('negative')
    def test_secret_create_with_invalid_payload_(self, payload):
        """Covers attempting to create secret with invalid payload types

        Tests the negative cases of invalid types (list and int).
        """
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.payload = payload

        self.assertRaises(
            exceptions.PayloadException,
            secret.store
        )

    @utils.parameterized_dataset({
        'empty': [''],
        'zero': [0]
    })
    @testcase.attr('negative')
    def test_secret_with_no_payload_exception(self, payload):
        """Covers creating secrets with various invalid payloads.

        These requests will fail with a value error before the request to the
        server is made
        """
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.payload = payload

        self.assertRaises(
            exceptions.PayloadException,
            secret.store
        )

    @utils.parameterized_dataset({
        'negative_five_long_expire': {
            'timezone': '-05:00',
            'days': 5},

        'positive_five_long_expire': {
            'timezone': '+05:00',
            'days': 5},

        'negative_one_short_expire': {
            'timezone': '-01',
            'days': 1},

        'positive_one_short_expire': {
            'timezone': '+01',
            'days': 1}
    })
    @testcase.attr('positive')
    def test_secret_create_defaults_valid_expiration(self, **kwargs):
        """Create secrets with a various valid expiration data."""

        timestamp = utils.create_timestamp_w_tz_and_offset(**kwargs)
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.expiration = timestamp

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertIsNotNone(resp)
        self.assertEqual(secret.name, resp.name)

    @utils.parameterized_dataset({
        'malformed_timezone': {
            'timezone': '-5:00',
            'days': 0}
    })
    @testcase.attr('negative')
    def test_secret_create_defaults_invalid_expiration(self, **kwargs):
        """Create secrets with various invalid expiration data."""
        timestamp = utils.create_timestamp_w_tz_and_offset(**kwargs)
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.expiration = timestamp

        e = self.assertRaises(
            exceptions.HTTPClientError,
            secret.store
        )

        self.assertEqual(400, e.status_code)

    @utils.parameterized_dataset({
        'text/plain':
            [
                'meowwwwwwwmeowwwwwww',
                'text/plain'],
        'application/octet-stream':
            [
                base64.b64encode(
                    b'F\x130\x89f\x8e\xd9\xa1\x0e\x1f\r\xf67uu\x8b'),
                'application/octet-stream'
            ]
    })
    @testcase.attr('positive')
    def test_secret_update_nones(self, payload, payload_content_type):
        """Cover case of updating with all nones in the Secret object."""
        secret = self.barbicanclient.secrets.create(**secret_create_nones_data)
        secret.payload = None
        secret.payload_content_type = None

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        secret.payload = payload
        secret.update()

        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertEqual(payload, resp.payload)
        self.assertEqual(payload_content_type, resp.payload_content_type)

    @utils.parameterized_dataset({
        'alphanumeric': ['1f34ds'],
        'punctuation': ['~!@#$%^&*()_+`-={}[]|:;<>,.?'],
        'uuid': ['54262d9d-4bc7-4821-8df0-dc2ca8e112bb'],
        'len_255': ['a' * 255],
        'empty': [''],
        'null': [None]
    })
    @testcase.attr('positive')
    def test_secret_get_defaults_metadata_w_valid_name(self, name):
        """Covers getting and checking a secret's metadata."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.name = name

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertEqual("ACTIVE", resp.status)
        self.assertEqual(name, resp.name)
        self.assertEqual(secret.mode, resp.mode)
        self.assertEqual(secret.algorithm, resp.algorithm)
        self.assertEqual(secret.bit_length, resp.bit_length)

    @utils.parameterized_dataset({
        'symmetric': ['symmetric',
                      'aes',
                      128,
                      ('\x00\x01\x02\x03\x04\x05\x06\x07'
                       '\x00\x01\x02\x03\x04\x05\x06\x07')],
        'private': ['private',
                    'rsa',
                    2048,
                    keys.get_private_key_pem()],
        'public': ['public',
                   'rsa',
                   2048,
                   keys.get_public_key_pem()],
        'certificate': ['certificate',
                        'rsa',
                        2048,
                        keys.get_certificate_pem()],
        'opaque': ['opaque',
                   None,
                   None,
                   (b'\x00\x01\x02\x03\x04\x05\x06\x07')],
        'passphrase': ['passphrase',
                       None,
                       None,
                       keys.get_passphrase_txt()],
    })
    @testcase.attr('positive')
    def test_secret_get_defaults_secret_type(self, secret_type, algorithm,
                                             bit_length, payload):
        """Covers getting and checking a secret's metadata."""
        secret = self.barbicanclient.secrets.create(
            **secret_create_defaults_data)
        secret.secret_type = secret_type
        secret.algorithm = algorithm
        secret.bit_length = bit_length
        # payload should not be encoded.
        secret.payload = payload

        secret_ref = self.cleanup.add_entity(secret)
        self.assertIsNotNone(secret_ref)

        resp = self.barbicanclient.secrets.get(secret_ref)
        self.assertEqual("ACTIVE", resp.status)
        self.assertEqual(secret_type, resp.secret_type)

    @utils.parameterized_dataset({
        'query_by_name': {
            'secret_1_dict': dict(name="name1"),
            'secret_2_dict': dict(name="name2"),
            'query_dict': dict(name="name1")
        },
        'query_by_algorithm': {
            'secret_1_dict': dict(algorithm="algorithm1"),
            'secret_2_dict': dict(algorithm="algorithm2"),
            'query_dict': dict(algorithm="algorithm1")
        },
        'query_by_mode': {
            'secret_1_dict': dict(mode="mode1"),
            'secret_2_dict': dict(mode="mode2"),
            'query_dict': dict(mode="mode1")
        },
        'query_by_bit_length': {
            'secret_1_dict': dict(bit_length=1024),
            'secret_2_dict': dict(bit_length=2048),
            'query_dict': dict(bits=1024)
        },
        'query_by_secret_type': {
            'secret_1_dict': dict(secret_type='opaque'),
            'secret_2_dict': dict(secret_type='symmetric'),
            'query_dict': dict(secret_type='opaque')
        },
    })
    @testcase.attr('positive')
    def test_secret_list_with_filter(self, secret_1_dict, secret_2_dict,
                                     query_dict):
        secret_1 = self.barbicanclient.secrets.create(**secret_1_dict)
        secret_1_ref = self.cleanup.add_entity(secret_1)
        self.assertIsNotNone(secret_1_ref)
        secret_2 = self.barbicanclient.secrets.create(**secret_2_dict)
        secret_2_ref = self.cleanup.add_entity(secret_2)
        self.assertIsNotNone(secret_2_ref)

        secret_list = self.barbicanclient.secrets.list(**query_dict)

        self.assertEqual(1, len(secret_list))

    @utils.parameterized_dataset({
        'query_by_name': {
            'secret_1_dict': dict(name="name1"),
            'secret_2_dict': dict(name="name2"),
            'sort': 'name'
        },
        'query_by_algorithm': {
            'secret_1_dict': dict(algorithm="algorithm1"),
            'secret_2_dict': dict(algorithm="algorithm2"),
            'sort': 'algorithm'
        },
        'query_by_mode': {
            'secret_1_dict': dict(mode="mode1"),
            'secret_2_dict': dict(mode="mode2"),
            'sort': 'mode'
        },
        'query_by_bit_length': {
            'secret_1_dict': dict(bit_length=1024),
            'secret_2_dict': dict(bit_length=2048),
            'sort': 'bit_length'
        },
        'query_by_secret_type': {
            'secret_1_dict': dict(secret_type='opaque'),
            'secret_2_dict': dict(secret_type='symmetric'),
            'sort': 'secret_type'
        },
    })
    @testcase.attr('positive')
    def test_secret_list_with_sort(self, secret_1_dict, secret_2_dict, sort):
        secret_1 = self.barbicanclient.secrets.create(**secret_1_dict)
        secret_1_ref = self.cleanup.add_entity(secret_1)
        self.assertIsNotNone(secret_1_ref)
        secret_2 = self.barbicanclient.secrets.create(**secret_2_dict)
        secret_2_ref = self.cleanup.add_entity(secret_2)
        self.assertIsNotNone(secret_2_ref)

        query_dict = {'sort': sort + ":asc"}
        secret_list = self.barbicanclient.secrets.list(**query_dict)

        self.assertEqual(2, len(secret_list))
        self.assertEqual(secret_1_ref, secret_list[0].secret_ref)

        query_dict = {'sort': sort + ":desc"}
        secret_list = self.barbicanclient.secrets.list(**query_dict)

        self.assertEqual(2, len(secret_list))
        self.assertEqual(secret_2_ref, secret_list[0].secret_ref)

    @utils.parameterized_dataset({
        'created': {
            'date_type': 'created',
        },
        'updated': {
            'date_type': 'updated',
        },
        'expiration': {
            'date_type': 'expiration',
        },
    })
    @testcase.attr('positive')
    def test_secret_list_with_date_filter(self, date_type):
        now = timeutils.utcnow()
        expiration_1 = (now + datetime.timedelta(days=3)).isoformat()
        expiration_2 = (now + datetime.timedelta(days=5)).isoformat()
        secret_1 = self.barbicanclient.secrets.create(expiration=expiration_1)
        secret_1_ref = self.cleanup.add_entity(secret_1)
        self.assertIsNotNone(secret_1_ref)
        payload = "gF6+lLoF3ohA9aPRpt+6bQ=="
        self.barbicanclient.secrets.update(secret_1_ref, payload)

        time.sleep(1)

        secret_2 = self.barbicanclient.secrets.create(expiration=expiration_2)
        secret_2_ref = self.cleanup.add_entity(secret_2)
        self.assertIsNotNone(secret_2_ref)

        time_to_search_1 = getattr(secret_1, date_type).isoformat()
        time_to_search_2 = getattr(secret_2, date_type).isoformat()

        # Search for secrets with secret 1's time
        query_dict = {date_type: time_to_search_1}
        secret_list = self.barbicanclient.secrets.list(**query_dict)

        self.assertEqual(1, len(secret_list))
        self.assertEqual(secret_1_ref, secret_list[0].secret_ref)

        # Search for secrets with time < secret 2, i.e. secret 1
        query_dict = {date_type: 'lt:' + time_to_search_2}
        secret_list = self.barbicanclient.secrets.list(**query_dict)

        self.assertEqual(1, len(secret_list))
        self.assertEqual(secret_1_ref, secret_list[0].secret_ref)

        # Search for secrets with time < secret 1, i.e. none
        query_dict = {date_type: 'lt:' + time_to_search_1}
        secret_list = self.barbicanclient.secrets.list(**query_dict)

        self.assertEqual(0, len(secret_list))

        # Search for secrets with time <= secret 2, i.e. both secrets
        query_dict = {date_type: 'lte:' + time_to_search_2}
        secret_list = self.barbicanclient.secrets.list(**query_dict)

        self.assertEqual(2, len(secret_list))

        # Search for secrets with time > secret 1, i.e. secret 2
        query_dict = {date_type: 'gt:' + time_to_search_1}
        secret_list = self.barbicanclient.secrets.list(**query_dict)

        self.assertEqual(1, len(secret_list))
        self.assertEqual(secret_2_ref, secret_list[0].secret_ref)

        # Search for secrets with time > secret 2, i.e. none
        query_dict = {date_type: 'gt:' + time_to_search_2}
        secret_list = self.barbicanclient.secrets.list(**query_dict)

        self.assertEqual(0, len(secret_list))

        # Search for secrets with time >= secret 1, i.e. both secrets
        query_dict = {date_type: 'gte:' + time_to_search_1}
        secret_list = self.barbicanclient.secrets.list(**query_dict)

        self.assertEqual(2, len(secret_list))

        # Sort secrets by date
        query_dict = {'sort': date_type + ":asc"}
        secret_list = self.barbicanclient.secrets.list(**query_dict)

        self.assertEqual(2, len(secret_list))
        self.assertEqual(secret_1_ref, secret_list[0].secret_ref)

        # Sort secrets by date
        query_dict = {'sort': date_type + ":desc"}
        secret_list = self.barbicanclient.secrets.list(**query_dict)

        self.assertEqual(2, len(secret_list))
        self.assertEqual(secret_2_ref, secret_list[0].secret_ref)



# Copyright (c) 2013 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import importlib
import logging
import os
import re
import sys
import warnings

from keystoneauth1 import adapter
from keystoneauth1 import session as ks_session
from oslo_utils import importutils

from barbicanclient import exceptions


LOG = logging.getLogger(__name__)
_DEFAULT_SERVICE_TYPE = 'key-manager'
_DEFAULT_SERVICE_INTERFACE = 'public'
_DEFAULT_API_VERSION = 'v1'
_SUPPORTED_API_VERSION_MAP = {'v1': 'barbicanclient.v1.client.Client'}


class _HTTPClient(adapter.Adapter):

    def __init__(self, session, microversion, project_id=None, **kwargs):
        endpoint = kwargs.pop('endpoint', None)
        if endpoint:
            kwargs['endpoint_override'] = "{}/{}/".format(
                endpoint.rstrip('/'),
                kwargs.get('version')
            )

        super().__init__(session, **kwargs)
        self.microversion = microversion

        if project_id is None:
            self._default_headers = dict()
        else:
            # If provided we'll include the project ID in all requests.
            self._default_headers = {'X-Project-Id': project_id}

    def _set_microversion(self, microversion):
        self.microversion = microversion
        self.default_microversion = microversion

    def _parse_version(self, version):
        """Parse microversion to tuple."""
        match = re.match(r"^([1-9]\d*)\.([1-9]\d*|0)$",
                         version)
        if match:
            ver_major = int(match.group(1))
            ver_minor = int(match.group(2))
        else:
            raise ValueError(
                f"Invalid version: {version}")
        return (ver_major, ver_minor)

    def is_supported_microversion(self, min_version, max_version=None):
        """Check if the microversion is supported.

        :param min_version: minimal version of API
        :param max_version: maximum version of API

        :returns: True if server supports minimal and maximum API version
             requirements. False in other case.
        """
        version = self._parse_version(self.microversion)
        minv = self._parse_version(min_version)

        if max_version is not None:
            maxv = self._parse_version(max_version)
            return (maxv >= version >= minv)

        return (version >= minv)

    def request(self, *args, **kwargs):
        headers = kwargs.setdefault('headers', {})
        headers.update(self._default_headers)

        # Set raise_exc=False by default so that we handle request exceptions
        kwargs.setdefault('raise_exc', False)

        resp = super(_HTTPClient, self).request(*args, **kwargs)
        self._check_status_code(resp)
        return resp

    def get(self, *args, **kwargs):
        headers = kwargs.setdefault('headers', {})
        headers.setdefault('Accept', 'application/json')

        return super(_HTTPClient, self).get(*args, **kwargs).json()

    def post(self, path, *args, **kwargs):
        path = self._fix_path(path)

        return super(_HTTPClient, self).post(path, *args, **kwargs).json()

    def _fix_path(self, path):
        if not path[-1] == '/':
            path += '/'
        return path

    def _get_raw(self, path, *args, **kwargs):
        return self.request(path, 'GET', *args, **kwargs).content

    def _check_status_code(self, resp):
        status = resp.status_code
        LOG.debug('Response status {0}'.format(status))
        if status == 401:
            LOG.error('Auth error: {0}'.format(self._get_error_message(resp)))
            raise exceptions.HTTPAuthError(
                '{0}'.format(self._get_error_message(resp))
            )
        if not status or status >= 500:
            LOG.error('5xx Server error: {0}'.format(
                self._get_error_message(resp)
            ))
            raise exceptions.HTTPServerError(
                '{0}'.format(self._get_error_message(resp)),
                status
            )
        if status >= 400:
            LOG.error('4xx Client error: {0}'.format(
                self._get_error_message(resp)
            ))
            raise exceptions.HTTPClientError(
                '{0}'.format(self._get_error_message(resp)),
                status
            )

    def _get_error_message(self, resp):
        try:
            response_data = resp.json()
            message = response_data['title']
            description = response_data.get('description')
            if description:
                message = '{0}: {1}'.format(message, description)
        except ValueError:
            message = resp.content
        return message


def Client(version=None, session=None, *args, **kwargs):
    """Barbican client used to interact with barbican service.

    :param session: An instance of keystoneauth1.session.Session that
        can be either authenticated, or not authenticated.  When using
        a non-authenticated Session, you must provide some additional
        parameters.  When no session is provided it will default to a
        non-authenticated Session. (optional)
    :param endpoint: Barbican endpoint url override. Required when a
        session is not given, or when using a non-authenticated session.
        When using an authenticated session, the client will attempt
        to get the endpoint from the Keystone service catalog. (optional)
    :param project_id: The project ID used for context in Barbican.
        Required when a session is not given, or when using a
        non-authenticated session.
        When using an authenticated session, the project ID will be
        provided by the authentication mechanism and this parameter
        will be ignored. (optional)
    :param verify: When a session is not given, the client will create
        a non-authenticated session.  This parameter is passed to the
        session that is created.  If set to False, it allows
        barbicanclient to perform "insecure" TLS (https) requests.
        The server's certificate will not be verified against any
        certificate authorities. (optional)
        WARNING: This option should be used with caution.
    :param version: Used as an endpoint filter when using an authenticated
        keystone session.  When using a non-authenticated keystone session,
        this value is appended to the required endpoint url override.
        Defaults to 'v1'.
    :param service_type: Used as an endpoint filter when using an
        authenticated keystone session.
        Defaults to 'key-manager'.
    :param service_name: Used as an endpoint filter when using an
        authenticated keystone session.
    :param interface: Used as an endpoint filter when using an
        authenticated keystone session. Defaults to 'public'.
    :param region_name: Used as an endpoint filter when using an
        authenticated keystone session.
    :param microversion: Specifiy an API Microversion to be used.
        Defaults to '1.1'.
    """
    LOG.debug("Creating Client object")

    if not session:
        session = ks_session.Session(verify=kwargs.pop('verify', True))

    if session.auth is None:
        if kwargs.get('auth') is None:
            if not kwargs.get('endpoint'):
                raise ValueError('Barbican endpoint url must be provided when'
                                 ' not using auth in the Keystone Session.')
            if kwargs.get('project_id') is None:
                raise ValueError('Project ID must be provided when not using '
                                 'auth in the Keystone Session')
        else:
            session.auth = kwargs['auth']

    kwargs['version'] = version or _DEFAULT_API_VERSION
    kwargs.setdefault('service_type', _DEFAULT_SERVICE_TYPE)
    kwargs.setdefault('interface', _DEFAULT_SERVICE_INTERFACE)

    try:
        client_path = _SUPPORTED_API_VERSION_MAP[kwargs['version']]
        client_class = importutils.import_class(client_path)
        return client_class(session=session, *args, **kwargs)
    except (KeyError, ValueError):
        supported_versions = ', '.join(_SUPPORTED_API_VERSION_MAP.keys())
        msg = ("Invalid client version %(version)s; must be one of: "
               "%(versions)s") % {'version': kwargs.get('version'),
                                  'versions': supported_versions}
        raise exceptions.UnsupportedVersion(msg)


def env(*vars, **kwargs):
    """Search for the first defined of possibly many env vars

    Returns the first environment variable defined in vars, or
    returns the default defined in kwargs.

    Source: Keystone's shell.py
    """
    for v in vars:
        value = os.environ.get(v, None)
        if value:
            return value
    return kwargs.get('default', '')


class _LazyImporter(object):
    def __init__(self, module):
        self._module = module

    def __getattr__(self, name):
        # This is only called until the import has been done.
        lazy_submodules = [
            'acls',
            'cas',
            'containers',
            'orders',
            'secrets',
        ]
        if name in lazy_submodules:
            warnings.warn("The %s module is moved to barbicanclient/v1 "
                          "directory, direct import of "
                          "barbicanclient.client.%s "
                          "will be deprecated. Please import "
                          "barbicanclient.v1.%s instead."
                          % (name, name, name))
            return importlib.import_module('barbicanclient.v1.%s' % name)

        # Return module attributes like __all__ etc.
        return getattr(self._module, name)


sys.modules[__name__] = _LazyImporter(sys.modules[__name__])



#   Copyright 2017 Huawei, Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Barbican Client Library Binding"""

import importlib
import sys
import warnings

import pbr.version

from barbicanclient.v1 import acls
from barbicanclient.v1 import cas
from barbicanclient.v1 import containers
from barbicanclient.v1 import orders
from barbicanclient.v1 import secrets


version_info = pbr.version.VersionInfo("python-barbicanclient")
__version__ = version_info.version_string()

__all__ = (
    'acls',
    'cas',
    'containers',
    'orders',
    'secrets',
)


class _LazyImporter(object):
    def __init__(self, module):
        self._module = module

    def __getattr__(self, name):
        # This is only called until the import has been done.
        lazy_submodules = [
            'acls',
            'cas',
            'containers',
            'orders',
            'secrets',
        ]
        if name in lazy_submodules:
            warnings.warn("The %s module is moved to barbicanclient/v1 "
                          "directory, direct import of barbicanclient.%s "
                          "will be deprecated. Please import "
                          "barbicanclient.v1.%s instead."
                          % (name, name, name))
            return importlib.import_module('barbicanclient.v1.%s' % name)

        # Return module attributes like __all__ etc.
        return getattr(self._module, name)


sys.modules[__name__] = _LazyImporter(sys.modules[__name__])



# Copyright (c) 2013 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils
from oslo_utils import timeutils
import uuid

from barbicanclient import base
from barbicanclient.tests import test_client
from barbicanclient.v1 import orders


class OrdersTestCase(test_client.BaseEntityResource):
    def setUp(self):
        self._setUp('orders', entity_id='d0460cc4-2876-4493-b7de-fc5c812883cc')

        self.secret_ref = (self.endpoint +
                           '/secrets/a2292306-6da0-4f60-bd8a-84fc8d692716')

        self.key_order_data = """{{
            "status": "ACTIVE",
            "secret_ref": "{0}",
            "updated": "2014-10-21T17:15:50.871596",
            "meta": {{
                "name": "secretname",
                "algorithm": "aes",
                "payload_content_type": "application/octet-stream",
                "mode": "cbc",
                "bit_length": 256,
                "expiration": "2015-02-28T19:14:44.180394"
            }},
            "created": "2014-10-21T17:15:50.824202",
            "type": "key",
            "order_ref": "{1}"
        }}""".format(self.secret_ref, self.entity_href)

        self.key_order_invalid_data = """{{
            "status": "ACTIVE",
            "secret_ref": "{0}",
            "updated": "2014-10-21T17:15:50.871596",
            "meta": {{
                "name": "secretname",
                "algorithm": "aes",
                "request_type":"invalid",
                "payload_content_type": "application/octet-stream",
                "mode": "cbc",
                "bit_length": 256,
                "expiration": "2015-02-28T19:14:44.180394"
            }},
            "created": "2014-10-21T17:15:50.824202",
            "type": "key",
            "order_ref": "{1}"
        }}""".format(self.secret_ref, self.entity_href)

        self.container_ref = (
            self.endpoint + '/containers/a2292306-6da0-4f60-bd8a-84fc8d692716')
        self.source_container_ref = (
            self.endpoint + '/containers/c6f20480-c1e5-442b-94a0-cb3b5e0cf179')

        self.cert_order_data = """{{
            "status": "ACTIVE",
            "container_ref": "{0}",
            "updated": "2014-10-21T17:15:50.871596",
            "meta": {{
                "name": "secretname",
                "subject_dn": "cn=server.example.com,o=example.com",
                "request_type": "stored-key",
                "container_ref": "{1}"
            }},
            "created": "2014-10-21T17:15:50.824202",
            "type": "certificate",
            "order_ref": "{2}"
        }}""".format(self.container_ref, self.source_container_ref,
                     self.entity_href)

        self.manager = self.client.orders

    def _get_order_args(self, order_data):
        order_args = jsonutils.loads(order_data)
        order_args.update(order_args.pop('meta'))
        order_args.pop('type')
        return order_args


class WhenTestingKeyOrders(OrdersTestCase):

    def test_should_include_errors_in_str(self):
        order_args = self._get_order_args(self.key_order_data)
        error_code = 500
        error_reason = 'Something is broken'
        order_obj = orders.KeyOrder(api=None, error_status_code=error_code,
                                    error_reason=error_reason, **order_args)
        self.assertIn(str(error_code), str(order_obj))
        self.assertIn(error_reason, str(order_obj))

    def test_should_include_order_ref_in_repr(self):
        order_args = self._get_order_args(self.key_order_data)
        order_obj = orders.KeyOrder(api=None, **order_args)
        self.assertIn('order_ref=' + self.entity_href, repr(order_obj))

    def test_should_be_immutable_after_submit(self):
        data = {'order_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        order = self.manager.create_key(
            name='name',
            algorithm='algorithm',
            payload_content_type='payload_content_type'
        )
        order_href = order.submit()

        self.assertEqual(self.entity_href, order_href)
        # Verify that attributes are immutable after store.
        attributes = [
            "name", "expiration", "algorithm", "bit_length", "mode",
            "payload_content_type"
        ]
        for attr in attributes:
            try:
                setattr(order, attr, "test")
                self.fail("didn't raise an ImmutableException exception")
            except base.ImmutableException:
                pass

    def test_should_submit_via_constructor(self):
        data = {'order_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        order = self.manager.create_key(
            name='name',
            algorithm='algorithm',
            payload_content_type='payload_content_type'
        )
        order_href = order.submit()

        self.assertEqual(self.entity_href, order_href)

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_base + '/',
                         self.responses.last_request.url)

        # Verify that correct information was sent in the call.
        order_req = jsonutils.loads(self.responses.last_request.text)
        self.assertEqual('name', order_req['meta']['name'])
        self.assertEqual('algorithm',
                         order_req['meta']['algorithm'])
        self.assertEqual('payload_content_type',
                         order_req['meta']['payload_content_type'])

    def test_should_submit_via_attributes(self):
        data = {'order_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        order = self.manager.create_key()
        order.name = 'name'
        order.algorithm = 'algorithm'
        order.payload_content_type = 'payload_content_type'
        order_href = order.submit()

        self.assertEqual(self.entity_href, order_href)

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_base + '/',
                         self.responses.last_request.url)

        # Verify that correct information was sent in the call.
        order_req = jsonutils.loads(self.responses.last_request.text)
        self.assertEqual('name', order_req['meta']['name'])
        self.assertEqual('algorithm',
                         order_req['meta']['algorithm'])
        self.assertEqual('payload_content_type',
                         order_req['meta']['payload_content_type'])

    def test_should_not_be_able_to_set_generated_attributes(self):
        order = self.manager.create_key()

        # Verify that generated attributes cannot be set.
        attributes = [
            "order_ref", "secret_ref", "created", "updated", "status",
            "error_status_code", "error_reason"
        ]
        for attr in attributes:
            try:
                setattr(order, attr, "test")
                self.fail("didn't raise an AttributeError exception")
            except AttributeError:
                pass

    def test_should_delete_from_object(self, order_ref=None):
        order_ref = order_ref or self.entity_href

        data = {'order_ref': order_ref}
        self.responses.post(self.entity_base + '/', json=data)
        self.responses.delete(self.entity_href, status_code=204)

        order = self.manager.create_key(
            name='name',
            algorithm='algorithm',
            payload_content_type='payload_content_type'
        )
        order_href = order.submit()

        self.assertEqual(order_ref, order_href)

        order.delete()

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_href, self.responses.last_request.url)

    def test_should_delete_from_object_using_stripped_uuid(self):
        bad_href = "http://badsite.com/" + self.entity_id
        self.test_should_delete_from_object(bad_href)

    def test_should_delete_from_object_using_only_uuid(self):
        self.test_should_delete_from_object(self.entity_id)


class WhenTestingAsymmetricOrders(OrdersTestCase):

    def test_should_be_immutable_after_submit(self):
        data = {'order_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        order = self.manager.create_asymmetric(
            name='name',
            algorithm='algorithm',
            payload_content_type='payload_content_type'
        )
        order_href = order.submit()

        self.assertEqual(self.entity_href, order_href)

        # Verify that attributes are immutable after store.
        attributes = [
            "name", "expiration", "algorithm", "bit_length", "pass_phrase",
            "payload_content_type"
        ]
        for attr in attributes:
            try:
                setattr(order, attr, "test")
                self.fail(
                    "{0} didn't raise an ImmutableException exception".format(
                        attr
                    )
                )
            except base.ImmutableException:
                pass

    def test_create_asymmetric_order_w_passphrase(self):
        data = {'order_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        passphrase = str(uuid.uuid4())
        order = orders.AsymmetricOrder(
            api=self.manager._api,
            name='name',
            algorithm='algorithm',
            payload_content_type='payload_content_type',
            passphrase=passphrase,
        )
        order_href = order.submit()
        self.assertEqual(self.entity_href, order_href)
        self.assertEqual(passphrase, order.pass_phrase)

    def test_create_asymmetric_order_w_legacy_pass_phrase_param(self):
        data = {'order_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        passphrase = str(uuid.uuid4())
        order = orders.AsymmetricOrder(
            api=self.manager._api,
            name='name',
            algorithm='algorithm',
            payload_content_type='payload_content_type',
            pass_phrase=passphrase,
        )
        order_href = order.submit()
        self.assertEqual(self.entity_href, order_href)
        self.assertEqual(passphrase, order.pass_phrase)


class WhenTestingOrderManager(OrdersTestCase):

    def test_should_get(self, order_ref=None):
        order_ref = order_ref or self.entity_href

        self.responses.get(self.entity_href, text=self.key_order_data)

        order = self.manager.get(order_ref=order_ref)
        self.assertIsInstance(order, orders.KeyOrder)
        self.assertEqual(self.entity_href, order.order_ref)

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_href, self.responses.last_request.url)

    def test_should_get_using_stripped_uuid(self):
        bad_href = "http://badsite.com/" + self.entity_id
        self.test_should_get(bad_href)

    def test_should_get_using_only_uuid(self):
        self.test_should_get(self.entity_id)

    def test_should_get_invalid_meta(self):
        self.responses.get(self.entity_href, text=self.key_order_invalid_data)

        # Verify checking for invalid meta fields.
        self.assertRaises(TypeError,
                          self.manager.get,
                          self.entity_href)

    def test_should_get_list(self):
        data = {"orders": [jsonutils.loads(self.key_order_data)
                           for _ in range(3)]}
        self.responses.get(self.entity_base, json=data)

        orders_list = self.manager.list(limit=10, offset=5)
        self.assertTrue(len(orders_list) == 3)
        self.assertIsInstance(orders_list[0], orders.KeyOrder)
        self.assertEqual(self.entity_href, orders_list[0].order_ref)

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_base,
                         self.responses.last_request.url.split('?')[0])

        # Verify that correct information was sent in the call.
        self.assertEqual(['10'], self.responses.last_request.qs['limit'])
        self.assertEqual(['5'], self.responses.last_request.qs['offset'])

    def test_should_delete(self, order_ref=None):
        order_ref = order_ref or self.entity_href
        self.responses.delete(self.entity_href, status_code=204)

        self.manager.delete(order_ref=order_ref)

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_href, self.responses.last_request.url)

    def test_should_delete_using_stripped_uuid(self):
        bad_href = "http://badsite.com/" + self.entity_id
        self.test_should_delete(bad_href)

    def test_should_delete_using_only_uuid(self):
        self.test_should_delete(self.entity_id)

    def test_should_fail_delete_no_href(self):
        self.assertRaises(ValueError, self.manager.delete, None)

    def test_should_get_total(self):
        self.responses.get(self.entity_base, json={'total': 1})
        total = self.manager.total()
        self.assertEqual(1, total)

    def test_get_formatted_data(self):
        self.responses.get(self.entity_href, text=self.key_order_data)

        order = self.manager.get(order_ref=self.entity_href)
        data = order._get_formatted_data()

        order_args = self._get_order_args(self.key_order_data)
        self.assertEqual(timeutils.parse_isotime(
                         order_args['created']).isoformat(),
                         data[4])



# Copyright (c) 2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import functools
import logging

from oslo_utils.timeutils import parse_isotime

from barbicanclient import base
from barbicanclient import formatter
from barbicanclient.v1 import acls as acl_manager
from barbicanclient.v1 import secrets as secret_manager


LOG = logging.getLogger(__name__)


def _immutable_after_save(func):
    @functools.wraps(func)
    def wrapper(self, *args):
        if hasattr(self, '_container_ref') and self._container_ref:
            raise base.ImmutableException()
        return func(self, *args)
    return wrapper


class ContainerFormatter(formatter.EntityFormatter):

    columns = ("Container href",
               "Name",
               "Created",
               "Status",
               "Type",
               "Secrets",
               "Consumers",
               )

    def _get_formatted_data(self):
        formatted_secrets = None
        formatted_consumers = None
        if self.secrets:
            formatted_secrets = '\n'.join((
                '='.join((name, secret_ref)) if name else secret_ref
                for name, secret_ref in self.secret_refs.items()
            ))
        if self.consumers:
            formatted_consumers = '\n'.join((str(c) for c in self.consumers))
        created = self.created.isoformat() if self.created else None
        data = (self.container_ref,
                self.name,
                created,
                self.status,
                self._type,
                formatted_secrets,
                formatted_consumers,
                )
        return data


class Container(ContainerFormatter):
    """Container is a generic grouping of Secrets"""
    _entity = 'containers'
    _type = 'generic'

    def __init__(self, api, name=None, secrets=None, consumers=None,
                 container_ref=None, created=None, updated=None, status=None,
                 secret_refs=None):
        self._api = api
        self._secret_manager = secret_manager.SecretManager(api)
        self._name = name
        self._container_ref = container_ref
        self._secret_refs = secret_refs
        self._cached_secrets = dict()
        self._initialize_secrets(secrets)
        if container_ref:
            self._consumers = consumers if consumers else list()
            self._created = parse_isotime(created) if created else None
            self._updated = parse_isotime(updated) if updated else None
            self._status = status
        else:
            self._consumers = list()
            self._created = None
            self._updated = None
            self._status = None
        self._acl_manager = acl_manager.ACLManager(api)
        self._acls = None

    def _initialize_secrets(self, secrets):
        try:
            self._fill_secrets_from_secret_refs()
        except Exception:
            raise ValueError("One or more of the provided secret_refs could "
                             "not be retrieved!")
        if secrets:
            try:
                for name, secret in secrets.items():
                    self.add(name, secret)
            except Exception:
                raise ValueError("One or more of the provided secrets are not "
                                 "valid Secret objects!")

    def _fill_secrets_from_secret_refs(self):
        if self._secret_refs:
            self._cached_secrets = dict(
                (name.lower() if name else "",
                 self._secret_manager.get(secret_ref=secret_ref))
                for name, secret_ref in self._secret_refs.items()
            )

    @property
    def container_ref(self):
        return self._container_ref

    @property
    def name(self):
        if self._container_ref and not self._name:
            self._reload()
        return self._name

    @property
    def created(self):
        return self._created

    @property
    def updated(self):
        return self._updated

    @property
    def status(self):
        if self._container_ref and not self._status:
            self._reload()
        return self._status

    @property
    def acls(self):
        """Get ACL settings for this container."""
        if self._container_ref and not self._acls:
            self._acls = self._acl_manager.get(self.container_ref)
        return self._acls

    @property
    def secret_refs(self):
        if self._cached_secrets:
            self._secret_refs = dict(
                (name, secret.secret_ref)
                for name, secret in self._cached_secrets.items()
            )

        return self._secret_refs

    @property
    def secrets(self, cache=True):
        """List of Secrets in Containers"""
        if not self._cached_secrets or not cache:
            self._fill_secrets_from_secret_refs()
        return self._cached_secrets

    @property
    def consumers(self):
        return self._consumers

    @name.setter
    @_immutable_after_save
    def name(self, value):
        self._name = value

    @_immutable_after_save
    def add(self, name, secret):
        if not isinstance(secret, secret_manager.Secret):
            raise ValueError("Must provide a valid Secret object")
        if name.lower() in self.secrets:
            raise KeyError("A secret with this name already exists!")
        self._cached_secrets[name.lower()] = secret

    @_immutable_after_save
    def remove(self, name):
        self._cached_secrets.pop(name.lower(), None)
        if self._secret_refs:
            self._secret_refs.pop(name.lower(), None)

    @_immutable_after_save
    def store(self):
        """Store Container in Barbican"""
        secret_refs = self._get_secrets_and_store_them_if_necessary()

        container_dict = base.filter_null_keys({
            'name': self.name,
            'type': self._type,
            'secret_refs': secret_refs
        })

        LOG.debug("Request body: {0}".format(container_dict))

        # Save, store container_ref and return
        response = self._api.post(self._entity, json=container_dict)
        if response:
            self._container_ref = response['container_ref']
        return self.container_ref

    def delete(self):
        """Delete container from Barbican"""
        if self._container_ref:
            uuid_ref = base.calculate_uuid_ref(self._container_ref,
                                               self._entity)
            self._api.delete(uuid_ref)
            self._container_ref = None
            self._status = None
            self._created = None
            self._updated = None
        else:
            raise LookupError("Secret is not yet stored.")

    def _get_secrets_and_store_them_if_necessary(self):
        # Save all secrets if they are not yet saved
        LOG.debug("Storing secrets: {0}".format(base.censored_copy(
                                                self.secrets, ['payload'])))
        secret_refs = []
        for name, secret in self.secrets.items():
            if secret and not secret.secret_ref:
                secret.store()
            secret_refs.append({'name': name, 'secret_ref': secret.secret_ref})
        return secret_refs

    def _reload(self):
        if not self._container_ref:
            raise AttributeError("container_ref not set, cannot reload data.")
        LOG.debug('Getting container - Container href: {0}'
                  .format(self._container_ref))
        uuid_ref = base.calculate_uuid_ref(self._container_ref,
                                           self._entity)
        try:
            response = self._api.get(uuid_ref)
        except AttributeError:
            raise LookupError('Container {0} could not be found.'
                              .format(self._container_ref))
        self._name = response.get('name')
        self._consumers = response.get('consumers', [])
        created = response.get('created')
        updated = response.get('updated')
        self._created = parse_isotime(created) if created else None
        self._updated = parse_isotime(updated) if updated else None
        self._status = response.get('status')

    def _get_named_secret(self, name):
        return self.secrets.get(name)

    def __repr__(self):
        return 'Container(name="{0}")'.format(self.name)


class RSAContainerFormatter(formatter.EntityFormatter):
    _get_generic_data = ContainerFormatter._get_formatted_data

    def _get_generic_columns(self):
        return ContainerFormatter.columns

    columns = ("Container href",
               "Name",
               "Created",
               "Status",
               "Type",
               "Public Key",
               "Private Key",
               "PK Passphrase",
               "Consumers",
               )

    def _get_formatted_data(self):
        formatted_public_key = None
        formatted_private_key = None
        formatted_pkp = None
        formatted_consumers = None
        if self.public_key:
            formatted_public_key = self.public_key.secret_ref
        if self.private_key:
            formatted_private_key = self.private_key.secret_ref
        if self.private_key_passphrase:
            formatted_pkp = self.private_key_passphrase.secret_ref
        if self.consumers:
            formatted_consumers = '\n'.join((str(c) for c in self.consumers))
        data = (self.container_ref,
                self.name,
                self.created,
                self.status,
                self._type,
                formatted_public_key,
                formatted_private_key,
                formatted_pkp,
                formatted_consumers,
                )
        return data


class RSAContainer(RSAContainerFormatter, Container):
    _required_secrets = ["public_key", "private_key"]
    _optional_secrets = ["private_key_passphrase"]
    _type = 'rsa'

    def __init__(self, api, name=None, public_key=None, private_key=None,
                 private_key_passphrase=None, consumers=[], container_ref=None,
                 created=None, updated=None, status=None, public_key_ref=None,
                 private_key_ref=None, private_key_passphrase_ref=None):
        secret_refs = {}
        if public_key_ref:
            secret_refs['public_key'] = public_key_ref
        if private_key_ref:
            secret_refs['private_key'] = private_key_ref
        if private_key_passphrase_ref:
            secret_refs['private_key_passphrase'] = private_key_passphrase_ref
        super(RSAContainer, self).__init__(
            api=api,
            name=name,
            consumers=consumers,
            container_ref=container_ref,
            created=created,
            updated=updated,
            status=status,
            secret_refs=secret_refs
        )
        if public_key:
            self.public_key = public_key
        if private_key:
            self.private_key = private_key
        if private_key_passphrase:
            self.private_key_passphrase = private_key_passphrase

    @property
    def public_key(self):
        """Secret containing the Public Key"""
        return self._get_named_secret("public_key")

    @property
    def private_key(self):
        """Secret containing the Private Key"""
        return self._get_named_secret("private_key")

    @property
    def private_key_passphrase(self):
        """Secret containing the Passphrase"""
        return self._get_named_secret("private_key_passphrase")

    @public_key.setter
    @_immutable_after_save
    def public_key(self, value):
        super(RSAContainer, self).remove("public_key")
        super(RSAContainer, self).add("public_key", value)

    @private_key.setter
    @_immutable_after_save
    def private_key(self, value):
        super(RSAContainer, self).remove("private_key")
        super(RSAContainer, self).add("private_key", value)

    @private_key_passphrase.setter
    @_immutable_after_save
    def private_key_passphrase(self, value):
        super(RSAContainer, self).remove("private_key_passphrase")
        super(RSAContainer, self).add("private_key_passphrase", value)

    def add(self, name, sec):
        raise NotImplementedError("`add()` is not implemented for "
                                  "Typed Containers")

    def __repr__(self):
        return 'RSAContainer(name="{0}")'.format(self.name)


class CertificateContainerFormatter(formatter.EntityFormatter):
    _get_generic_data = ContainerFormatter._get_formatted_data

    def _get_generic_columns(self):
        return ContainerFormatter.columns

    columns = ("Container href",
               "Name",
               "Created",
               "Status",
               "Type",
               "Certificate",
               "Intermediates",
               "Private Key",
               "PK Passphrase",
               "Consumers",
               )

    def _get_formatted_data(self):
        formatted_certificate = None
        formatted_private_key = None
        formatted_pkp = None
        formatted_intermediates = None
        formatted_consumers = None
        if self.certificate:
            formatted_certificate = self.certificate.secret_ref
        if self.intermediates:
            formatted_intermediates = self.intermediates.secret_ref
        if self.private_key:
            formatted_private_key = self.private_key.secret_ref
        if self.private_key_passphrase:
            formatted_pkp = self.private_key_passphrase.secret_ref
        if self.consumers:
            formatted_consumers = '\n'.join((str(c) for c in self.consumers))
        data = (self.container_ref,
                self.name,
                self.created,
                self.status,
                self._type,
                formatted_certificate,
                formatted_intermediates,
                formatted_private_key,
                formatted_pkp,
                formatted_consumers,
                )
        return data


class CertificateContainer(CertificateContainerFormatter, Container):
    _required_secrets = ["certificate", "private_key"]
    _optional_secrets = ["private_key_passphrase", "intermediates"]
    _type = 'certificate'

    def __init__(self, api, name=None, certificate=None, intermediates=None,
                 private_key=None, private_key_passphrase=None, consumers=[],
                 container_ref=None, created=None, updated=None, status=None,
                 certificate_ref=None, intermediates_ref=None,
                 private_key_ref=None, private_key_passphrase_ref=None):
        secret_refs = {}
        if certificate_ref:
            secret_refs['certificate'] = certificate_ref
        if intermediates_ref:
            secret_refs['intermediates'] = intermediates_ref
        if private_key_ref:
            secret_refs['private_key'] = private_key_ref
        if private_key_passphrase_ref:
            secret_refs['private_key_passphrase'] = private_key_passphrase_ref
        super(CertificateContainer, self).__init__(
            api=api,
            name=name,
            consumers=consumers,
            container_ref=container_ref,
            created=created,
            updated=updated,
            status=status,
            secret_refs=secret_refs
        )
        if certificate:
            self.certificate = certificate
        if intermediates:
            self.intermediates = intermediates
        if private_key:
            self.private_key = private_key
        if private_key_passphrase:
            self.private_key_passphrase = private_key_passphrase

    @property
    def certificate(self):
        """Secret containing the certificate"""
        return self._get_named_secret("certificate")

    @property
    def private_key(self):
        """Secret containing the private key"""
        return self._get_named_secret("private_key")

    @property
    def private_key_passphrase(self):
        """Secret containing the passphrase"""
        return self._get_named_secret("private_key_passphrase")

    @property
    def intermediates(self):
        """Secret containing intermediate certificates"""
        return self._get_named_secret("intermediates")

    @certificate.setter
    @_immutable_after_save
    def certificate(self, value):
        super(CertificateContainer, self).remove("certificate")
        super(CertificateContainer, self).add("certificate", value)

    @private_key.setter
    @_immutable_after_save
    def private_key(self, value):
        super(CertificateContainer, self).remove("private_key")
        super(CertificateContainer, self).add("private_key", value)

    @private_key_passphrase.setter
    @_immutable_after_save
    def private_key_passphrase(self, value):
        super(CertificateContainer, self).remove("private_key_passphrase")
        super(CertificateContainer, self).add("private_key_passphrase", value)

    @intermediates.setter
    @_immutable_after_save
    def intermediates(self, value):
        super(CertificateContainer, self).remove("intermediates")
        super(CertificateContainer, self).add("intermediates", value)

    def add(self, name, sec):
        raise NotImplementedError("`add()` is not implemented for "
                                  "Typed Containers")

    def __repr__(self):
        return 'CertificateContainer(name="{0}")'.format(self.name)


class ContainerManager(base.BaseEntityManager):
    """EntityManager for Container entities

    You should use the ContainerManager exposed by the Client and should not
    need to instantiate your own.
    """

    _container_map = {
        'generic': Container,
        'rsa': RSAContainer,
        'certificate': CertificateContainer
    }

    def __init__(self, api):
        super(ContainerManager, self).__init__(api, 'containers')

    def get(self, container_ref):
        """Retrieve an existing Container from Barbican

        :param container_ref: Full HATEOAS reference to a Container, or a UUID
        :returns: Container object or a subclass of the appropriate type
        """
        LOG.debug('Getting container - Container href: {0}'
                  .format(container_ref))
        uuid_ref = base.calculate_uuid_ref(container_ref, self._entity)
        try:
            response = self._api.get(uuid_ref)
        except AttributeError:
            raise LookupError('Container {0} could not be found.'
                              .format(container_ref))
        return self._generate_typed_container(response)

    def _generate_typed_container(self, response):
        resp_type = response.get('type', '').lower()
        container_type = self._container_map.get(resp_type)
        if not container_type:
            raise TypeError('Unknown container type "{0}".'
                            .format(resp_type))

        name = response.get('name')
        consumers = response.get('consumers', [])
        container_ref = response.get('container_ref')
        created = response.get('created')
        updated = response.get('updated')
        status = response.get('status')
        secret_refs = self._translate_secret_refs_from_json(
            response.get('secret_refs')
        )

        if container_type is RSAContainer:
            public_key_ref = secret_refs.get('public_key')
            private_key_ref = secret_refs.get('private_key')
            private_key_pass_ref = secret_refs.get('private_key_passphrase')
            return RSAContainer(
                api=self._api,
                name=name,
                consumers=consumers,
                container_ref=container_ref,
                created=created,
                updated=updated,
                status=status,
                public_key_ref=public_key_ref,
                private_key_ref=private_key_ref,
                private_key_passphrase_ref=private_key_pass_ref,
            )
        elif container_type is CertificateContainer:
            certificate_ref = secret_refs.get('certificate')
            intermediates_ref = secret_refs.get('intermediates')
            private_key_ref = secret_refs.get('private_key')
            private_key_pass_ref = secret_refs.get('private_key_passphrase')
            return CertificateContainer(
                api=self._api,
                name=name,
                consumers=consumers,
                container_ref=container_ref,
                created=created,
                updated=updated,
                status=status,
                certificate_ref=certificate_ref,
                intermediates_ref=intermediates_ref,
                private_key_ref=private_key_ref,
                private_key_passphrase_ref=private_key_pass_ref,
            )
        return container_type(
            api=self._api,
            name=name,
            secret_refs=secret_refs,
            consumers=consumers,
            container_ref=container_ref,
            created=created,
            updated=updated,
            status=status
        )

    @staticmethod
    def _translate_secret_refs_from_json(json_refs):
        return dict(
            (ref_pack.get('name'), ref_pack.get('secret_ref'))
            for ref_pack in json_refs
        )

    def create(self, name=None, secrets=None):
        """Factory method for `Container` objects

        `Container` objects returned by this method have not yet been
        stored in Barbican.

        :param name: A friendly name for the Container
        :param secrets: Secrets to populate when creating a Container
        :returns: Container
        :rtype: :class:`barbicanclient.v1.containers.Container`
        :raises barbicanclient.exceptions.HTTPAuthError: 401 Responses
        :raises barbicanclient.exceptions.HTTPClientError: 4xx Responses
        :raises barbicanclient.exceptions.HTTPServerError: 5xx Responses
        """
        return Container(
            api=self._api,
            name=name,
            secrets=secrets
        )

    def create_rsa(self, name=None, public_key=None, private_key=None,
                   private_key_passphrase=None):
        """Factory method for `RSAContainer` objects

        `RSAContainer` objects returned by this method have not yet been
        stored in Barbican.

        :param name: A friendly name for the RSAContainer
        :param public_key: Secret object containing a Public Key
        :param private_key: Secret object containing a Private Key
        :param private_key_passphrase: Secret object containing a passphrase
        :returns: RSAContainer
        :rtype: :class:`barbicanclient.v1.containers.RSAContainer`
        :raises barbicanclient.exceptions.HTTPAuthError: 401 Responses
        :raises barbicanclient.exceptions.HTTPClientError: 4xx Responses
        :raises barbicanclient.exceptions.HTTPServerError: 5xx Responses
        """
        return RSAContainer(
            api=self._api,
            name=name,
            public_key=public_key,
            private_key=private_key,
            private_key_passphrase=private_key_passphrase
        )

    def create_certificate(self, name=None, certificate=None,
                           intermediates=None, private_key=None,
                           private_key_passphrase=None):
        """Factory method for `CertificateContainer` objects

        `CertificateContainer` objects returned by this method have not yet
        been stored in Barbican.

        :param name: A friendly name for the CertificateContainer
        :param certificate: Secret object containing a Certificate
        :param intermediates: Secret object containing Intermediate Certs
        :param private_key: Secret object containing a Private Key
        :param private_key_passphrase: Secret object containing a passphrase
        :returns: CertificateContainer
        :rtype: :class:`barbicanclient.v1.containers.CertificateContainer`
        :raises barbicanclient.exceptions.HTTPAuthError: 401 Responses
        :raises barbicanclient.exceptions.HTTPClientError: 4xx Responses
        :raises barbicanclient.exceptions.HTTPServerError: 5xx Responses
        """
        return CertificateContainer(
            api=self._api,
            name=name,
            certificate=certificate,
            intermediates=intermediates,
            private_key=private_key,
            private_key_passphrase=private_key_passphrase
        )

    def delete(self, container_ref):
        """Delete a Container from Barbican

        :param container_ref: Full HATEOAS reference to a Container, or a UUID
        :raises barbicanclient.exceptions.HTTPAuthError: 401 Responses
        :raises barbicanclient.exceptions.HTTPClientError: 4xx Responses
        :raises barbicanclient.exceptions.HTTPServerError: 5xx Responses
        """
        if not container_ref:
            raise ValueError('container_ref is required.')
        uuid_ref = base.calculate_uuid_ref(container_ref, self._entity)
        self._api.delete(uuid_ref)

    def list(self, limit=10, offset=0, name=None, type=None):
        """List containers for the project.

        This method uses the limit and offset
        parameters for paging.

        :param limit: Max number of containers returned
        :param offset: Offset containers to begin list
        :param name: Name filter for the list
        :param type: Type filter for the list
        :returns: list of Container metadata objects
        :raises barbicanclient.exceptions.HTTPAuthError: 401 Responses
        :raises barbicanclient.exceptions.HTTPClientError: 4xx Responses
        :raises barbicanclient.exceptions.HTTPServerError: 5xx Responses
        """
        LOG.debug('Listing containers - offset {0} limit {1} name {2} type {3}'
                  .format(offset, limit, name, type))
        params = {'limit': limit, 'offset': offset}
        if name:
            params['name'] = name
        if type:
            params['type'] = type

        response = self._api.get(self._entity, params=params)

        return [self._generate_typed_container(container)
                for container in response.get('containers', [])]

    def register_consumer(self, container_ref, name, url):
        """Add a consumer to the container

        :param container_ref: Full HATEOAS reference to a Container, or a UUID
        :param name: Name of the consuming service
        :param url: URL of the consuming resource
        :returns: A container object per the get() method
        :raises barbicanclient.exceptions.HTTPAuthError: 401 Responses
        :raises barbicanclient.exceptions.HTTPClientError: 4xx Responses
        :raises barbicanclient.exceptions.HTTPServerError: 5xx Responses
        """
        LOG.debug('Creating consumer registration for container '
                  '{0} as {1}: {2}'.format(container_ref, name, url))
        container_uuid = base.validate_ref_and_return_uuid(
            container_ref, 'Container')
        href = '{0}/{1}/consumers'.format(self._entity, container_uuid)
        consumer_dict = dict()
        consumer_dict['name'] = name
        consumer_dict['URL'] = url

        response = self._api.post(href, json=consumer_dict)
        return self._generate_typed_container(response)

    def remove_consumer(self, container_ref, name, url):
        """Remove a consumer from the container

        :param container_ref: Full HATEOAS reference to a Container, or a UUID
        :param name: Name of the previously consuming service
        :param url: URL of the previously consuming resource
        :raises barbicanclient.exceptions.HTTPAuthError: 401 Responses
        :raises barbicanclient.exceptions.HTTPClientError: 4xx Responses
        :raises barbicanclient.exceptions.HTTPServerError: 5xx Responses
        """
        LOG.debug('Deleting consumer registration for container '
                  '{0} as {1}: {2}'.format(container_ref, name, url))
        container_uuid = base.validate_ref_and_return_uuid(
            container_ref, 'Container')
        href = '{0}/{1}/consumers'.format(self._entity, container_uuid)
        consumer_dict = {
            'name': name,
            'URL': url
        }

        self._api.delete(href, json=consumer_dict)



# Copyright (c) 2015 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import functools
import logging

from oslo_utils.timeutils import parse_isotime

from barbicanclient import base
from barbicanclient import formatter


LOG = logging.getLogger(__name__)


def lazy(func):
    @functools.wraps(func)
    def wrapper(self, *args):
        self._fill_lazy_properties()
        return func(self, *args)
    return wrapper


class CAFormatter(formatter.EntityFormatter):

    columns = ("CA href",
               "Name",
               "Description",
               "Created",
               "Updated",
               "Status",
               "Plugin Name",
               "Plugin CA ID",
               "Expiration"
               )

    def _get_formatted_data(self):
        created = self.created.isoformat() if self.created else None
        updated = self.updated.isoformat() if self.updated else None
        expiration = self.expiration.isoformat() if self.expiration else None
        data = (self.ca_ref,
                self.name,
                self.description,
                created,
                updated,
                self.status,
                self.plugin_name,
                self.plugin_ca_id,
                expiration
                )
        return data


class CA(CAFormatter):
    """Certificate authority

    CAs represent certificate authorities or subCAs with which the Barbican
    service is configured to interact.
    """
    _entity = 'cas'

    def __init__(self, api, meta=None, expiration=None,
                 plugin_name=None, plugin_ca_id=None,
                 ca_ref=None, created=None, updated=None,
                 status=None, creator_id=None):
        """Certificate authority

        CA objects should not be instantiated directly.  You should use
        the `create` or `get` methods of the
        :class:`barbicanclient.cas.CAManager` instead.
        """
        self._api = api
        self._ca_ref = ca_ref
        self._fill_from_data(
            meta=meta,
            expiration=expiration,
            plugin_name=plugin_name,
            plugin_ca_id=plugin_ca_id,
            created=created,
            updated=updated,
            status=status,
            creator_id=creator_id
        )

    @property
    def ca_ref(self):
        return self._ca_ref

    @property
    @lazy
    def name(self):
        return self._name

    @property
    @lazy
    def expiration(self):
        return self._expiration

    @property
    @lazy
    def description(self):
        return self._description

    @property
    @lazy
    def plugin_name(self):
        return self._plugin_name

    @property
    @lazy
    def plugin_ca_id(self):
        return self._plugin_ca_id

    @property
    @lazy
    def created(self):
        return self._created

    @property
    @lazy
    def updated(self):
        return self._updated

    @property
    @lazy
    def status(self):
        return self._status

    def _fill_from_data(self, meta=None, expiration=None,
                        plugin_name=None, plugin_ca_id=None, created=None,
                        updated=None, status=None, creator_id=None):
        self._name = None
        self._description = None
        if meta:
            for s in meta:
                key = list(s.keys())[0]
                value = list(s.values())[0]
                if key == 'name':
                    self._name = value
                if key == 'description':
                    self._description = value
        self._plugin_name = plugin_name
        self._plugin_ca_id = plugin_ca_id
        self._expiration = expiration
        self._creator_id = creator_id
        if self._expiration:
            self._expiration = parse_isotime(self._expiration)
        if self._ca_ref:
            self._status = status
            self._created = created
            self._updated = updated
            if self._created:
                self._created = parse_isotime(self._created)
            if self._updated:
                self._updated = parse_isotime(self._updated)
        else:
            self._status = None
            self._created = None
            self._updated = None

    def _fill_lazy_properties(self):
        if self._ca_ref and not self._plugin_name:
            uuid_ref = base.calculate_uuid_ref(self._ca_ref, self._entity)
            result = self._api.get(uuid_ref)
            self._fill_from_data(
                meta=result.get('meta'),
                expiration=result.get('expiration'),
                plugin_name=result.get('plugin_name'),
                plugin_ca_id=result.get('plugin_ca_id'),
                created=result.get('created'),
                updated=result.get('updated'),
                status=result.get('status')
            )

    def __repr__(self):
        if self._ca_ref:
            return 'CA(ca_ref="{0}")'.format(self._ca_ref)
        return 'CA(name="{0}")'.format(self._name)


class CAManager(base.BaseEntityManager):
    """Entity Manager for Secret entities"""

    def __init__(self, api):
        super(CAManager, self).__init__(api, 'cas')

    def get(self, ca_ref):
        """Retrieve an existing CA from Barbican

        :param str ca_ref: Full HATEOAS reference to a CA or UUID
        :returns: CA object retrieved from Barbican
        :rtype: :class:`barbicanclient.v1.cas.CA`
        :raises barbicanclient.exceptions.HTTPAuthError: 401 Responses
        :raises barbicanclient.exceptions.HTTPClientError: 4xx Responses
        :raises barbicanclient.exceptions.HTTPServerError: 5xx Responses
        """
        LOG.debug("Getting ca - CA href: {0}".format(ca_ref))
        base.validate_ref_and_return_uuid(ca_ref, 'CA')
        return CA(
            api=self._api,
            ca_ref=ca_ref
        )

    def list(self, limit=10, offset=0, name=None):
        """List CAs for the project

        This method uses the limit and offset parameters for paging,
        and also supports filtering.

        :param limit: Max number of CAs returned
        :param offset: Offset secrets to begin list
        :param name: Name filter for the list
        :returns: list of CA objects that satisfy the provided filter
            criteria.
        :rtype: list
        :raises barbicanclient.exceptions.HTTPAuthError: 401 Responses
        :raises barbicanclient.exceptions.HTTPClientError: 4xx Responses
        :raises barbicanclient.exceptions.HTTPServerError: 5xx Responses
        """
        LOG.debug('Listing CAs - offset {0} limit {1}'.format(offset, limit))
        params = {'limit': limit, 'offset': offset}
        if name:
            params['name'] = name

        response = self._api.get(self._entity, params=params)

        return [
            CA(api=self._api, ca_ref=s)
            for s in response.get('cas', [])
        ]



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from testtools import testcase

from functionaltests.client import base
from functionaltests.common import cleanup
from functionaltests import utils


order_create_key_data = {
    "name": "barbican functional test secret name",
    "algorithm": "aes",
    "bit_length": 256,
    "mode": "cbc",
    "payload_content_type": "application/octet-stream",
}

# Any field with None will be created in the model with None as the value
# but will be omitted in the final request (via the requests package)
# to the server.
#
# Given that fact, order_create_nones_data is effectively an empty json request
# to the server.
order_create_nones_data = {
    'type': None,
    "meta": {
        "name": None,
        "algorithm": None,
        "bit_length": None,
        "mode": None,
        "payload_content_type": None,
    }
}


@utils.parameterized_test_case
class OrdersTestCase(base.TestCase):

    def setUp(self):
        super(OrdersTestCase, self).setUp()
        self.cleanup = cleanup.CleanUp(self.barbicanclient)

    def tearDown(self):
        self.cleanup.delete_all_entities()
        super(OrdersTestCase, self).tearDown()

    @testcase.attr('positive')
    def test_create_order_defaults(self):
        """Covers simple order creation."""
        order = self.barbicanclient.orders.create_key(
            **order_create_key_data)
        order_ref = self.cleanup.add_entity(order)

        self.assertIsNotNone(order_ref)

    @testcase.attr('positive')
    def test_get_order_defaults_metadata(self):
        """Covers order metadata.

        Assumes that the order status will be active or pending.
        """

        # first create an order
        order = self.barbicanclient.orders.create_key(
            **order_create_key_data)
        order_ref = self.cleanup.add_entity(order)

        # verify that the order was created successfully
        self.assertIsNotNone(order_ref)

        # given the order href, retrieve the order
        order_resp = self.barbicanclient.orders.get(order_ref)

        # verify that the get was successful
        self.assertTrue(order_resp.status == "ACTIVE" or
                        order_resp.status == "PENDING")

        # verify the metadata
        self.assertEqual(order.name,
                         order_resp.name)
        self.assertEqual(order.mode,
                         order_resp.mode)
        self.assertEqual(order.algorithm,
                         order_resp.algorithm)
        self.assertEqual(order.bit_length,
                         order_resp.bit_length)
        self.assertEqual(order.payload_content_type,
                         order_resp.payload_content_type)

    @testcase.attr('positive')
    def test_get_order_defaults(self):
        """Covers getting an order.

        Assumes that the order status will be active or pending.
        """

        # create an order
        order = self.barbicanclient.orders.create_key(
            **order_create_key_data)
        order_ref = self.cleanup.add_entity(order)
        self.assertIsNotNone(order_ref)

        # get the order
        order_resp = self.barbicanclient.orders.get(order_ref)

        # verify the order
        self.assertIsNotNone(order_resp.order_ref)
        self.assertEqual('key', order_resp._type)
        self.assertTrue(order_resp.status == "ACTIVE" or
                        order_resp.status == "PENDING")

        if order_resp.status == "ACTIVE":
            self.assertIsNotNone(order_resp.secret_ref)

    @testcase.attr('positive')
    def test_delete_order_defaults(self):
        """Covers simple order deletion."""

        # create an order
        order = self.barbicanclient.orders.create_key(
            **order_create_key_data)
        order_ref = order.submit()
        secret_ref = self.barbicanclient.orders.get(order_ref).secret_ref

        # delete the order
        delete_resp = self.barbicanclient.orders.delete(order_ref)
        self.assertIsNone(delete_resp)

        # delete the secret
        self.barbicanclient.secrets.delete(secret_ref)

    @testcase.attr('positive')
    def test_get_orders_defaults(self):
        """Covers getting a list of orders."""
        limit = 7
        offset = 0
        total = 10

        # create the orders
        for i in range(0, total + 1):
            order = self.barbicanclient.orders.create_key(
                **order_create_key_data)
            order_ref = self.cleanup.add_entity(order)
            self.assertIsNotNone(order_ref)

        # get a list of orders
        orders_list = self.barbicanclient.orders.list(limit=limit,
                                                      offset=offset)

        # verify that the get for the list was successful
        self.assertEqual(limit, len(orders_list))



# Copyright (c) 2016 GohighSec, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from keystoneauth1 import discover
from keystoneauth1.exceptions.http import NotAcceptable

from barbicanclient import client as base_client
from barbicanclient.v1 import acls
from barbicanclient.v1 import cas
from barbicanclient.v1 import containers
from barbicanclient.v1 import orders
from barbicanclient.v1 import secrets


LOG = logging.getLogger(__name__)
_SUPPORTED_MICROVERSIONS = [(1, 0),
                            (1, 1),
                            (1, 2)]
# For microversion 1.0, API status is "stable"
_STABLE = "STABLE"


class Client(object):

    def __init__(self, session=None, *args, **kwargs):
        """Barbican client implementation for API version v1

        This class is dynamically loaded by the factory function
        `barbicanclient.client.Client`.  It's recommended to use that
        function instead of making instances of this class directly.
        """
        microversion = self._get_normalized_microversion(
            kwargs.pop('microversion', None))
        normalized_microversion = self._get_max_supported_version(
            session,
            kwargs.get('endpoint'),
            kwargs.get('version'),
            kwargs.get('service_type'),
            kwargs.get('service_name'),
            kwargs.get('interface'),
            kwargs.get('region_name'),
            microversion)

        if normalized_microversion is None:
            raise ValueError(
                "Endpoint does not support selected microversion"
            )
        kwargs['default_microversion'] = normalized_microversion

        # TODO(dmendiza): This should be a private member
        self.client = base_client._HTTPClient(
            session, normalized_microversion, *args, **kwargs)

        self.secrets = secrets.SecretManager(self.client)
        self.orders = orders.OrderManager(self.client)
        self.containers = containers.ContainerManager(self.client)
        self.cas = cas.CAManager(self.client)
        self.acls = acls.ACLManager(self.client)

    def _set_microversion(self, microversion):
        self.client._set_microversion(microversion)

    def _get_normalized_microversion(self, microversion):
        if microversion is None:
            return

        # We need to make sure that the microversion is something we understand
        normalized = discover.normalize_version_number(microversion)
        if normalized not in _SUPPORTED_MICROVERSIONS:
            raise ValueError(
                "Invalid microversion {}: Microversion requested is not "
                "supported by the client".format(microversion))
        return discover.version_to_string(normalized)

    def _get_max_supported_version(self, session, endpoint, version,
                                   service_type, service_name, interface,
                                   region_name, microversion):
        min_ver, max_ver = self._get_min_max_server_supported_microversion(
            session, endpoint, version, service_type, service_name, interface,
            region_name)

        if microversion is None:
            for client_version in _SUPPORTED_MICROVERSIONS[::-1]:
                if discover.version_between(min_ver, max_ver, client_version):
                    return self._get_normalized_microversion(client_version)
            raise ValueError(
                "Couldn't find a version supported by both client and server")

        if discover.version_between(min_ver, max_ver, microversion):
            return microversion

        raise ValueError(
            "Invalid microversion {}: Microversion requested is not "
            "supported by the server".format(microversion))

    def _get_min_max_server_supported_microversion(self, session, endpoint,
                                                   version, service_type,
                                                   service_name, interface,
                                                   region_name):
        if not endpoint:
            endpoint = session.get_endpoint(
                service_type=service_type,
                service_name=service_name,
                interface=interface,
                region_name=region_name,
                version=version
            )

        return self._get_min_max_version(session, endpoint, '1.1')

    def _get_min_max_version(self, session, endpoint, microversion):
        try:
            # If the microversion requested in the version_header is outside of
            # the range of microversions supported, return 406 Not Acceptable.
            resp = discover.get_version_data(
                session, endpoint,
                version_header='key-manager ' + microversion)
        except NotAcceptable:
            return None, None

        resp = resp[0]
        status = resp['status'].upper()
        if status == _STABLE:
            # status is only set to STABLE in two cases
            # 1. when the server is older and is ignoring the microversion
            #    header
            # 2. when we ask for microversion 1.0 and the server
            #    understands the header
            # in either case min/max will be 1.0
            min_ver = '1.0'
            max_ver = '1.0'
        else:
            # any other status will have a min/max
            min_ver = resp['min_version']
            max_ver = resp['max_version']
        return min_ver, max_ver
